
/***************************************************************************** 
 *  Copyright 1998 National Instruments Corporation.  All Rights Reserved.   * 
 *****************************************************************************/

/*****************************************************************************
 *  California Instruments RP Series Instrument Driver                               
 *  LabWindows/CVI 5.0 Instrument Driver                                     
 *  Original Release: 8/19/98                                  
 *  By: CI, California Instruments                              
 *      PH. (619) 677-9040   Fax (619) 677-0940                              
 *                                                                           
 *  Modification History:                                                    
 *                                                                           
 *  8/19/98 - Instrument Driver Created.                  
 *  2/12/99 - Add 2001RP extensions.
 *	6/07/00 - Add 2003RP functions
 *
 *	3/1/2005 v1.0.0.1 - Added delays (30mS) for every commands send to the source.
 *					  - Corrected Reset function(added commands before sending *RST).
 *****************************************************************************/
#include <userint.h>    
#include <utility.h>
#include <string.h>
#include <stdio.h>  
#include <formatio.h>
#include <math.h>
#include <ctype.h>
#include <stdlib.h>
#include <visa.h>
#include <gpib.h>
#include <rs232.h>
#include "CIC_RP50.h"

/*****************************************************************************
 *--------------------- Hidden Attribute Declarations -----------------------*
 *****************************************************************************/

/*****************************************************************************
 *---------------------------- Useful Macros --------------------------------*
 *****************************************************************************/

    /*- I/O buffer size -----------------------------------------------------*/
#define BUFFER_SIZE                             512L        

    /*- 488.2 Event Status Register (ESR) Bits ------------------------------*/
#define IEEE_488_2_QUERY_ERROR_BIT              0x04
#define IEEE_488_2_DEVICE_DEPENDENT_ERROR_BIT   0x08
#define IEEE_488_2_EXECUTION_ERROR_BIT          0x10
#define IEEE_488_2_COMMAND_ERROR_BIT            0x20

    /*- List of channels passed to the Ivi_BuildChannelTable function -------*/ 
#define CHANNEL_LIST                            "1"

	/*  Variables declaration -----------------------------------------------*/
static ViReal64 freq_low_limit;
static ViReal64 freq_high_limit;
static ViReal64 volt_high_limit;
static ViReal64 curr_high_limit;
static ViInt32 model_number;
static ViInt16 serial_flag;
static int i;
static char Serial_Name[5];

/*****************************************************************************
 *-------------- Utility Function Declarations (Non-Exported) ---------------*
 *****************************************************************************/
static ViStatus CIC_RP50_InitAttributes (ViSession vi);
static ViStatus CIC_RP50_DefaultInstrSetup (ViSession openInstrSession);
static ViStatus CIC_RP50_CheckStatus (ViSession vi);
static ViStatus CIC_RP50_WaitForOPC (ViSession vi, ViInt32 maxTime);

static ViStatus CIC_RP50_GetOutputStatus (ViSession vi, ViConstString channel,
                                          ViInt32 function, ViReal64 *value);

static ViStatus CIC_RP50_SimulateMeasurement (ViSession vi,
                                              ViConstString channel,
                                              ViInt32 readingArraySize,
                                              ViReal64 readingArray[],
                                              ViInt32 *pointsRead,
                                              ViBoolean multipoint);

static ViStatus CIC_RP50_GetDriverSetupOption (ViSession vi,
                                               ViConstString optionTag,
                                               ViInt32 optionValueSize,
                                               ViChar optionValue[]);

    /*- File I/O Utility Functions -*/
static ViStatus CIC_RP50_ReadToFile (ViSession vi, ViConstString filename, 
                                     ViInt32 maxBytesToRead, ViInt32 fileAction, ViInt32 *totalBytesWritten);
static ViStatus CIC_RP50_WriteFromFile (ViSession vi, ViConstString filename, 
                                        ViInt32 maxBytesToWrite, ViInt32 byteOffset, 
                                        ViInt32 *totalBytesWritten);

/*****************************************************************************
 *----------------- Callback Declarations (Non-Exported) --------------------*
 *****************************************************************************/

    /*- Global Session Callbacks --------------------------------------------*/
    
static ViStatus _VI_FUNC CIC_RP50_CheckStatusCallback (ViSession vi, ViSession io);

    /*- Attribute callbacks -------------------------------------------------*/

static ViStatus _VI_FUNC CIC_RP50AttrDriverRevision_ReadCallback (ViSession vi,
                                                                  ViSession io, 
                                                                  ViConstString channelName,
                                                                  ViAttr attributeId, 
                                                                  const ViConstString cacheValue);

static ViStatus _VI_FUNC CIC_RP50AttrIdQueryResponse_ReadCallback (ViSession vi,
                                                                   ViSession io, 
                                                                   ViConstString channelName, 
                                                                   ViAttr attributeId, 
                                                                   const ViConstString cacheValue);

static ViStatus _VI_FUNC CIC_RP50AttrVoltageLimit_ReadCallback (ViSession vi, 
                                                                ViSession io, 
                                                                ViConstString channelName, 
                                                                ViAttr attributeId, 
                                                                ViReal64 *value);
                                                                
static ViStatus _VI_FUNC CIC_RP50AttrCurrentLimit_ReadCallback (ViSession vi, 
                                                                ViSession io, 
                                                                ViConstString channelName, 
                                                                ViAttr attributeId, 
                                                                ViReal64 *value);

static ViStatus _VI_FUNC CIC_RP50AttrOutputConnected_CheckCallback (ViSession vi,
                                                                    ViConstString channelName,
                                                                    ViAttr attributeId,
                                                                    ViBoolean value);

static ViStatus _VI_FUNC CIC_RP50AttrOutputConnected_WriteCallback (ViSession vi, 
                                                                    ViSession io, 
                                                                    ViConstString channelName, 
                                                                    ViAttr attributeId, 
                                                                    ViBoolean value);

static ViStatus _VI_FUNC CIC_RP50AttrOutputConnected_ReadCallback (ViSession vi, 
                                                                   ViSession io, 
                                                                   ViConstString channelName, 
                                                                   ViAttr attributeId, 
                                                                   ViBoolean *value);

static ViStatus _VI_FUNC CIC_RP50AttrFrequency_WriteCallback (ViSession vi, 
                                                              ViSession io, 
                                                              ViConstString channelName, 
                                                              ViAttr attributeId, 
                                                              ViReal64 value);

static ViStatus _VI_FUNC CIC_RP50AttrFrequency_ReadCallback (ViSession vi, 
                                                             ViSession io, 
                                                             ViConstString channelName, 
                                                             ViAttr attributeId, 
                                                             ViReal64 *value);

static ViStatus _VI_FUNC CIC_RP50AttrACVAmplitude_WriteCallback (ViSession vi, 
                                                                 ViSession io, 
                                                                 ViConstString channelName, 
                                                                 ViAttr attributeId, 
                                                                 ViReal64 value);

static ViStatus _VI_FUNC CIC_RP50AttrACVAmplitude_ReadCallback (ViSession vi, 
                                                                ViSession io, 
                                                                ViConstString channelName, 
                                                                ViAttr attributeId, 
                                                                ViReal64 *value);

static ViStatus _VI_FUNC CIC_RP50AttrACCAmplitude_WriteCallback (ViSession vi, 
                                                                 ViSession io, 
                                                                 ViConstString channelName, 
                                                                 ViAttr attributeId, 
                                                                 ViReal64 value);

static ViStatus _VI_FUNC CIC_RP50AttrACCAmplitude_ReadCallback (ViSession vi, 
                                                                ViSession io, 
                                                                ViConstString channelName, 
                                                                ViAttr attributeId, 
                                                                ViReal64 *value);

static ViStatus _VI_FUNC CIC_RP50AttrFrequencyLowLimit_ReadCallback (ViSession vi,
                                                                     ViSession io,
                                                                     ViConstString channelName,
                                                                     ViAttr attributeId,
                                                                     ViReal64 *value);

static ViStatus _VI_FUNC CIC_RP50AttrFrequencyHighLimit_ReadCallback (ViSession vi,
                                                                      ViSession io,
                                                                      ViConstString channelName,
                                                                      ViAttr attributeId,
                                                                      ViReal64 *value);

static ViStatus _VI_FUNC CIC_RP50AttrQueryFunction_ReadCallback (ViSession vi,
                                                                 ViSession io,
                                                                 ViConstString channelName,
                                                                 ViAttr attributeId,
                                                                 ViInt32 *value);

static ViStatus _VI_FUNC CIC_RP50AttrQueryFunction_WriteCallback (ViSession vi,
                                                                  ViSession io,
                                                                  ViConstString channelName,
                                                                  ViAttr attributeId,
                                                                  ViInt32 value);

static ViStatus _VI_FUNC CIC_RP50AttrACVRange_ReadCallback (ViSession vi,
                                                            ViSession io,
                                                            ViConstString channelName,
                                                            ViAttr attributeId,
                                                            ViReal64 *value);

static ViStatus _VI_FUNC CIC_RP50AttrACVRange_WriteCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViReal64 value);

static ViStatus _VI_FUNC CIC_RP50AttrRemoteOnOff_CheckCallback (ViSession vi,
                                                                ViConstString channelName,
                                                                ViAttr attributeId,
                                                                ViInt32 value);

static ViStatus _VI_FUNC CIC_RP50AttrRemoteOnOff_ReadCallback (ViSession vi,
                                                               ViSession io,
                                                               ViConstString channelName,
                                                               ViAttr attributeId,
                                                               ViInt32 *value);

static ViStatus _VI_FUNC CIC_RP50AttrRemoteOnOff_WriteCallback (ViSession vi,
                                                                ViSession io,
                                                                ViConstString channelName,
                                                                ViAttr attributeId,
                                                                ViInt32 value);

static ViStatus _VI_FUNC CIC_RP50AttrDisplayMode_CheckCallback (ViSession vi,
                                                                ViConstString channelName,
                                                                ViAttr attributeId,
                                                                ViInt32 value);

static ViStatus _VI_FUNC CIC_RP50AttrDisplayMode_ReadCallback (ViSession vi,
                                                               ViSession io,
                                                               ViConstString channelName,
                                                               ViAttr attributeId,
                                                               ViInt32 *value);

static ViStatus _VI_FUNC CIC_RP50AttrDisplayMode_WriteCallback (ViSession vi,
                                                                ViSession io,
                                                                ViConstString channelName,
                                                                ViAttr attributeId,
                                                                ViInt32 value);

/*****************************************************************************
 *------------ User-Callable Functions (Exportable Functions) ---------------*
 *****************************************************************************/

/*****************************************************************************
 * Function: CIC_RP50_init   
 * Purpose:  VXIplug&play required function.  Calls the   
 *           CIC_RP50_InitWithOptions function.   
 *****************************************************************************/
ViStatus _VI_FUNC CIC_RP50_init (ViRsrc resourceName, ViBoolean IDQuery,
                                 ViBoolean resetDevice,
                                 ViSession *newVi, ViInt32 baudRate)  
{
    ViStatus    error = VI_SUCCESS;

    if (newVi == VI_NULL)
        {
        Ivi_SetErrorInfo (VI_NULL, VI_FALSE, IVI_ERROR_INVALID_PARAMETER, 
                          VI_ERROR_PARAMETER4, "Null address for Instrument Handle");
        checkErr( IVI_ERROR_INVALID_PARAMETER);
        }

    checkErr( CIC_RP50_InitWithOptions (resourceName, IDQuery, resetDevice, 
                                        "", newVi, baudRate));

Error:  
    return error;
}

/*****************************************************************************
 * Function: CIC_RP50_InitWithOptions                                       
 * Purpose:  This function creates a new IVI session and calls the 
 *           IviInit function.                                     
 *****************************************************************************/
ViStatus _VI_FUNC CIC_RP50_InitWithOptions (ViRsrc resourceName,
                                            ViBoolean IDQuery,
                                            ViBoolean resetDevice,
                                            ViString optionString,
                                            ViSession *newVi,
                                            ViInt32 baudRate)   

{
    ViStatus    error = VI_SUCCESS;
    ViSession   vi = VI_NULL;
    
    if (newVi == VI_NULL)
        {
        Ivi_SetErrorInfo (VI_NULL, VI_FALSE, IVI_ERROR_INVALID_PARAMETER, 
                          VI_ERROR_PARAMETER5, "Null address for Instrument Handle");
        checkErr( IVI_ERROR_INVALID_PARAMETER);
        }

    *newVi = VI_NULL;
    
        /* create a new interchangeable driver */
    checkErr( Ivi_SpecificDriverNew ("CIC_RP50", optionString, &vi));  
    
        /* init the driver */
    checkErr( CIC_RP50_IviInit (resourceName, IDQuery, resetDevice, vi, baudRate)); 
    *newVi = vi;
    
Error:
    if (error < VI_SUCCESS) 
        Ivi_Dispose (vi);
        
    return error;
}

/*****************************************************************************
 * Function: CIC_RP50_IviInit                                                       
 * Purpose:  This function is called by CIC_RP50_InitWithOptions  
 *           or by an IVI class driver.  This function initializes the I/O 
 *           interface, optionally resets the device, optionally performs an
 *           ID query, and sends a default setup to the instrument.                
 *****************************************************************************/
ViStatus _VI_FUNC CIC_RP50_IviInit (ViRsrc resourceName, ViBoolean IDQuery,
                                    ViBoolean reset, ViSession vi, ViInt32 baudRate)

{
    ViStatus    error = VI_SUCCESS;
    ViSession   io = VI_NULL;
    ViInt32 n;
    
    
    checkErr( Ivi_BuildChannelTable (vi, CHANNEL_LIST, VI_FALSE, VI_NULL));
    
     /* Add attributes */
    checkErr( CIC_RP50_InitAttributes (vi));

   	if( !Ivi_Simulating(vi) ){
       	ViSession   rmSession = VI_NULL;
		
        /* Open instrument session */
        checkErr( Ivi_GetAttributeViSession (vi, VI_NULL, IVI_ATTR_VISA_RM_SESSION, 0, &rmSession) ); 
        viCheckErr( viOpen (rmSession, resourceName, VI_NULL, 5000, &io) );
        /* io session owned by driver now */
        checkErr( Ivi_SetAttributeViSession (vi, VI_NULL, IVI_ATTR_IO_SESSION, 0, io) );  
		
		/* Add for RS-232 */
		#ifdef VI_ATTR_ASRL_DATA_BITS
    	/*- Configure serial COM, baud rate 9600, data bit 8, parity 0, stop 1	*/
	    for( i = 0 ; i < 4 ; i++ )
			Serial_Name[i] = resourceName[i];

	    Serial_Name[4] = '\0';
     	if( (CompareStrings (Serial_Name, 0, "ASRL", 0, 0)) == 0 ){        /* ASRL */
	      
		   	serial_flag = 1;  
		   	viCheckErr( viSetAttribute(io, VI_ATTR_TMO_VALUE, 10000) );
		   	viCheckErr( viSetAttribute(io, VI_ATTR_ASRL_BAUD, baudRate) ); 
	       	viCheckErr( viSetAttribute(io, VI_ATTR_ASRL_DATA_BITS, 8) ); 
	       	viCheckErr( viSetAttribute(io, VI_ATTR_ASRL_PARITY, VI_ASRL_PAR_NONE) ); 
	       	viCheckErr( viSetAttribute(io, VI_ATTR_ASRL_STOP_BITS, VI_ASRL_STOP_ONE) ); 
	       	viCheckErr( viSetAttribute(io, VI_ATTR_ASRL_FLOW_CNTRL, VI_ASRL_FLOW_RTS_CTS) );
	   	}																	
		else{
		   	serial_flag = 0;
		   	viCheckErr( viSetAttribute(io, VI_ATTR_TMO_VALUE, 5000) );
		}
        
        #endif
	    /* Configure VISA Formatted I/O */
	     
		viCheckErr( viSetBuf (io, VI_READ_BUF | VI_WRITE_BUF, 4000) );
		viCheckErr( viSetAttribute (io, VI_ATTR_WR_BUF_OPER_MODE, VI_FLUSH_ON_ACCESS) );
		viCheckErr( viSetAttribute (io, VI_ATTR_RD_BUF_OPER_MODE, VI_FLUSH_ON_ACCESS) );
		
		CIC_RP50_RemoteOnOff( vi, "1", 0 ); //v1.0.0.3
       	CIC_RP50_RemoteOnOff( vi, "1", 1 ); //v1.0.0.3
       	
       	/*- Reset instrument ----------------------------------------------------*/
       	if( reset ) 
         	checkErr( CIC_RP50_reset (vi) );
       	else  /*- Send Default Instrument Setup ---------------------------------*/
         	checkErr( CIC_RP50_DefaultInstrSetup (vi));
	
	   	/*- Identification Query ------------------------------------------------*/
	   	if( IDQuery ){
	     	ViChar rdBuffer[BUFFER_SIZE];
	
	    #define VALID_RESPONSE_STRING_START        "CI"
	
	    checkErr( Ivi_GetAttributeViString (vi, VI_NULL, CIC_RP50_ATTR_ID_QUERY_RESPONSE, 
	                                        0, BUFFER_SIZE, rdBuffer));
	    /* collect model number */
	    n = Scan( rdBuffer, "%s>%s[dt#]%d", &model_number ); 
	    
	    /* check for ID */
	    	if( strncmp (rdBuffer, VALID_RESPONSE_STRING_START, 
	        	         strlen(VALID_RESPONSE_STRING_START)) != 0 ){             
	        	viCheckErr( VI_ERROR_FAIL_ID_QUERY);
	    	}
	    }
		
		/* collect limit values */
	
        viCheckParm( Ivi_GetAttributeViReal64  (vi, "1",
                                                CIC_RP50_ATTR_VOLTAGE_LIMIT,
                                                0, &volt_high_limit),
                                                3, "Voltage Limit");
        
        viCheckParm( Ivi_GetAttributeViReal64  (vi, "1",
                                                CIC_RP50_ATTR_CURRENT_LIMIT,
                                                0, &curr_high_limit),
                                                4, "Current Limit"); 
                                               
        viCheckParm( Ivi_GetAttributeViReal64  (vi, "1",
                                                CIC_RP50_ATTR_FREQUENCY_LOW_LIMIT,
                                                0, &freq_low_limit),
                                                4, "Frequency Low Limit");
        
        viCheckParm( Ivi_GetAttributeViReal64  (vi, "1",
                                                CIC_RP50_ATTR_FREQUENCY_HIGH_LIMIT,
                                                0, &freq_high_limit),
                                                4, "Frequency High Limit");
        }
    checkErr( CIC_RP50_CheckStatus (vi));

Error:
    if( error < VI_SUCCESS ){
       	if( !Ivi_Simulating (vi) && io )
           	viClose( io );
   	}
    return error;
}

/*****************************************************************************
 * Function: CIC_RP50_close                                                           
 * Purpose:  This function closes the instrument.                            
 *
 *           Note:  This function must unlock the session before calling
 *           Ivi_Dispose.
 *****************************************************************************/
ViStatus _VI_FUNC CIC_RP50_close (ViSession vi)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Ivi_LockSession (vi, VI_NULL));

	if (!Ivi_Simulating(vi))                /* call only when locked */
	    {
	    ViSession   io = Ivi_IOSession(vi); /* call only when locked */
	    
	    checkErr( Ivi_SetNeedToCheckStatus (vi, VI_TRUE));
	    if (serial_flag)
	      {
	       viCheckErr( viPrintf (io, "SYST:LOC\n"));
	      }
	    else
	      viCheckErr( viPrintf (io, "SYST:LOC")); 
	    }
	    
	    Delay (.03);  
    checkErr( Ivi_LockSession (vi, VI_NULL)); 
    checkErr( CIC_RP50_IviClose (vi));

Error:    
    Ivi_UnlockSession (vi, VI_NULL);
    Ivi_Dispose (vi);  

    return error;
}

/*****************************************************************************
 * Function: CIC_RP50_IviClose                                                        
 * Purpose:  This function performs all of the drivers clean-up operations   
 *           except for closing the IVI session.  This function is called by 
 *           CIC_RP50_close or by an IVI class driver. 
 *
 *           Note:  This function must close the I/O session and set 
 *           IVI_ATTR_IO_SESSION to 0.
 *****************************************************************************/
ViStatus _VI_FUNC CIC_RP50_IviClose (ViSession vi)
{
    ViStatus error = VI_SUCCESS;
    ViSession io = VI_NULL;

        /* Do not lock here.  The caller manages the lock. */

    checkErr( Ivi_GetAttributeViSession (vi, VI_NULL, IVI_ATTR_IO_SESSION, 0, &io));

Error:
    Ivi_SetAttributeViSession (vi, VI_NULL, IVI_ATTR_IO_SESSION, 0, VI_NULL);
    if(io)                                                      
        {
        viClose (io);
        }
        
    return error;   
}

/*****************************************************************************
 * Function: CIC_RP50_QueryOutputFunction                                                   
 * Purpose:  This function queries the instrument output function and returns   
 *           the result.                                                     
 *****************************************************************************/
ViStatus _VI_FUNC CIC_RP50_QueryOutputFunction (ViSession vi,
                                                ViConstString channel,
                                                ViInt32 function, 
                                                ViReal64 *value)
{
	return CIC_RP50_GetOutputStatus (vi, channel, function, value);

}

/*****************************************************************************
 * Function: CIC_RP50_QueryOutputRelay                                                  
 * Purpose:  This function queries the instrument output relay status and 
 *			 returns the result.                                                     
 *****************************************************************************/
ViStatus _VI_FUNC CIC_RP50_QueryOutputRelay (ViSession vi,
                                             ViConstString channel,
                                             ViBoolean *status)
{
	ViStatus    error = VI_SUCCESS;

    checkErr( Ivi_LockSession (vi, VI_NULL));

	if (!Ivi_Simulating(vi))                /* call only when locked */
	    {
	    ViSession   io = Ivi_IOSession(vi); /* call only when locked */
	    
	    checkErr( Ivi_SetNeedToCheckStatus (vi, VI_TRUE));
	    if (serial_flag)
	      {
	       viCheckErr( viPrintf (io, "OUTP?\n"));
	       Delay (.05);
	       viCheckErr( viScanf (io, "%d\n", status)); 
	       Delay (.05);  
	      }
	    else
	      {
	       viCheckErr( viPrintf (io, "OUTP?"));
	       viCheckErr( viScanf (io, "%d", status)); 
	      }
	    }
	
	checkErr( CIC_RP50_DefaultInstrSetup (vi));                                
	
	checkErr( CIC_RP50_CheckStatus (vi));  
	Delay (.05);

Error:
    Ivi_UnlockSession (vi, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: CIC_RP50_SaveRegister                                                         
 * Purpose:  This function stores the present state of the instrument to a
 *    		 specified location in memory.                          
 *****************************************************************************/
ViStatus _VI_FUNC CIC_RP50_SaveRegister (ViSession vi,
                                         ViConstString channel,
                                         ViInt32 registerNumber)
{
	ViStatus	error = VI_SUCCESS;
	checkErr( Ivi_LockSession (vi, VI_NULL));

    if (!Ivi_Simulating(vi))                /* call only when locked */
	    {
	    ViSession   io = Ivi_IOSession(vi); /* call only	*/
    	if (serial_flag)
    	  {
    	   viCheckErr (viPrintf (io, "*SAV %ld\n", registerNumber));
    	  }
    	else
    	  viCheckErr (viPrintf (io, "*SAV %ld", registerNumber)); 
    	}
    	Delay (.03);  
    	
    checkErr( CIC_RP50_CheckStatus (vi));
Error:
	return error;
}

/*****************************************************************************
 * Function: CIC_RP50_RecallRegister                                                         
 * Purpose:  This function restores the instrument to a state that previously 
 *    		 saved in a specified location in memory.                          
 *****************************************************************************/
ViStatus _VI_FUNC CIC_RP50_RecallRegister (ViSession vi,
                                           ViConstString channel,
                                           ViInt32 registerNumber)
{
	ViStatus	error = VI_SUCCESS;
	checkErr( Ivi_LockSession (vi, VI_NULL));

   	if (!Ivi_Simulating(vi))                /* call only when locked */
	    {
	    ViSession   io = Ivi_IOSession(vi); /* call only */
    	if (serial_flag)
    	  {
    	   viCheckErr (viPrintf (io, "*RCL %ld\n", registerNumber));   
    	  }
    	else
    	  viCheckErr (viPrintf (io, "*RCL %ld", registerNumber));   
    	}
    	Delay (.03);  
    
    checkErr( CIC_RP50_CheckStatus (vi));
Error:
	return error;
}

/*****************************************************************************
 * Function: CIC_RP50_PowerOnRegister                                                       
 * Purpose:  This function defines the register value the instrument will use
 *           to initialize its parameters at power up.                 
 *****************************************************************************/
ViStatus _VI_FUNC CIC_RP50_PowerOnRegister (ViSession vi,
                                            ViConstString channel,
                                            ViInt32 registerNumber)
{
	ViStatus	error = VI_SUCCESS;
	checkErr( Ivi_LockSession (vi, VI_NULL));
	if (!Ivi_Simulating(vi))                /* call only when locked */
	    {
	    ViSession   io = Ivi_IOSession(vi); /* call only	*/
    	if (serial_flag)
    	  viCheckErr (viPrintf (io, "SYST:PON %ld\n", registerNumber)); 
    	else
    	  viCheckErr (viPrintf (io, "SYST:PON %ld", registerNumber));
    	}
    	
    	Delay (.03);  
    checkErr( CIC_RP50_CheckStatus (vi));
Error:
	return error;
}

/*****************************************************************************
 * Function: CIC_RP50_PowerOnQuery                                                       
 * Purpose:  This function queries the register value the instrument will use
 *           to initialize its parameters at power up.                 
 *****************************************************************************/
ViStatus _VI_FUNC CIC_RP50_PowerOnQuery (ViSession vi,
                                         ViConstString channel,
                                         ViInt32 *registerNumber)
{
	ViStatus	error = VI_SUCCESS;
	checkErr( Ivi_LockSession (vi, VI_NULL));
	if (!Ivi_Simulating(vi))                /* call only when locked */
	    {
	    ViSession   io = Ivi_IOSession(vi); /* call only */
    	if (serial_flag)
    	  {
    	   viCheckErr (viPrintf (io, "SYST:PON?\n"));
    	   viCheckErr (viScanf (io, "%ld\n", registerNumber));  
    	  }
    	else
    	  {
    	   viCheckErr (viPrintf (io, "SYST:PON?"));
    	   viCheckErr (viScanf (io, "%ld", registerNumber));  
    	  }
    	}
    	
    checkErr( CIC_RP50_CheckStatus (vi));
Error:
	return error;
}

/*****************************************************************************
 * Function: CIC_RP50_LoadInitialConfiguration                                                 
 * Purpose:  This function loads the initial values to the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC CIC_RP50_LoadInitialConfiguration (ViSession vi,
                                                     ViReal64 voltageRange,
                                                     ViReal64 frequency,
                                                     ViReal64 initialVoltage,
                                                     int outputRelay)
{
	ViStatus	error = VI_SUCCESS;

    checkErr( Ivi_LockSession (vi, VI_NULL));

	if (!Ivi_Simulating(vi))                /* call only when locked */
	   {
	    ViSession   io = Ivi_IOSession(vi); /* call only when locked */
	    
	    checkErr( Ivi_SetNeedToCheckStatus (vi, VI_TRUE));
	    
	    CIC_RP50_GenerateACVoltageRange(vi, "1", voltageRange);
	    CIC_RP50_GenerateACVoltage(vi, "1", initialVoltage);  
	    CIC_RP50_GenerateACFrequency(vi, "1", frequency);  
		if (outputRelay == 1)
		  CIC_RP50_ConnectOutput(vi, "1");  
		else
		  CIC_RP50_DisconnectOutput(vi, "1"); 
		
		CIC_RP50_SaveRegister(vi, "1", 7); 
		CIC_RP50_PowerOnRegister(vi, "1", 7);  
	   }	  
	checkErr( CIC_RP50_CheckStatus (vi));                                      
	Delay (.05); 
	
Error:
    Ivi_UnlockSession (vi, VI_NULL);
	return error;
}

/*****************************************************************************
 * Function: CIC_RP50_LoadSerialNumber                                                      
 * Purpose:  This function loads the serial number to the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC CIC_RP50_LoadSerialNumber (ViSession vi,
                                             ViConstString channelName,
                                             ViChar serialNumber[])
{
	ViStatus	error = VI_SUCCESS;
	ViChar      system_serial[25];  
	
    checkErr( Ivi_LockSession (vi, VI_NULL));

	if (!Ivi_Simulating(vi))                /* call only when locked */
	    {
	    ViSession   io = Ivi_IOSession(vi); /* call only when locked */
	    
	    checkErr( Ivi_SetNeedToCheckStatus (vi, VI_TRUE));
	    Fmt(system_serial, "%s<SYST:SNUM '%s'", serialNumber);  
	    if(serial_flag)
	      {
	       viCheckErr( viPrintf (io, system_serial));
	      }
	    else
	      viCheckErr( viPrintf (io, system_serial)); 
	    }
		Delay(.03);  
		
	checkErr( CIC_RP50_DefaultInstrSetup (vi));                                
	
	checkErr( CIC_RP50_CheckStatus (vi));                                      
	Delay (.05); 
Error:
    Ivi_UnlockSession (vi, VI_NULL);
	return error;
}

/*****************************************************************************
 * Function: CIC_RP50_SerialID                                                  
 * Purpose:  This function checks the serial ID to the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC CIC_RP50_Serial_ID (ViSession vi,
                                      ViInt32 *serialID)
{
	ViStatus	error = VI_SUCCESS;
	ViUInt32    retCnt;   
	unsigned char  	rdBuffer[BUFFER_SIZE];
	int n;
	
	checkErr( Ivi_LockSession (vi, VI_NULL) );

	if( !Ivi_Simulating(vi) ){                /* call only when locked */
	 	ViSession   io = Ivi_IOSession(vi); /* call only when locked */
	    
	    if( serial_flag ){
         	viCheckErr( viPrintf(io, "*IDN?\n") );
           	Delay (.05);
        }
        else
        	viCheckErr( viPrintf(io, "*IDN?") );
    
       	viCheckErr( viRead(io, rdBuffer, BUFFER_SIZE-1, &retCnt) );
       
       	/* collect model number */
	   	n = Scan( rdBuffer, "%s[i8]>%d", &serialID ); 	
	}
    
	checkErr( CIC_RP50_CheckStatus (vi) );                                      
	Delay(.03); 

Error:
    Ivi_UnlockSession( vi, VI_NULL );
	return error;
}

/*****************************************************************************
 * Function: CIC_RP50_Model_Number                                                  
 * Purpose:  This function scans the model number of the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC CIC_RP50_Model_Number (ViSession vi,
                                      ViChar model_number[])
{

	ViStatus	error = VI_SUCCESS;
	ViChar      rdBuffer[BUFFER_SIZE];
	ViUInt32    retCnt; 
	char data[5], buffer[20];
	int i, n, index;
	
	checkErr( Ivi_LockSession (vi, VI_NULL) );

	if (!Ivi_Simulating(vi)){                /* call only when locked */
	 	ViSession   io = Ivi_IOSession(vi); /* call only when locked */
	    if (serial_flag){
         	viCheckErr( viPrintf(io, "*IDN?\n") );
           	Delay (.05);
        }
        else
        	viCheckErr( viPrintf(io, "*IDN?") );
    
       	viCheckErr( viRead(io, rdBuffer, BUFFER_SIZE-1, &retCnt) );
        
      	/* scan model number */
    	index = FindPattern (rdBuffer, 0, -1, "CI,", 0, 0) + 3;
		n = Scan( rdBuffer, "%s[i*]>%s[t44]", index, model_number );
  	}
    
	checkErr( CIC_RP50_CheckStatus(vi) );                                      
	Delay(.03); 

Error:
    Ivi_UnlockSession( vi, VI_NULL );
	return error;
}																	

/*****************************************************************************
 * Function: CIC_RP50_Serial_Number                                                  
 * Purpose:  This function scans the serial number of the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC CIC_RP50_Serial_Number( ViSession vi,
											ViChar serial_number[] )
{
	ViStatus	error = VI_SUCCESS;
	ViChar      rdBuffer[BUFFER_SIZE];
	ViUInt32    retCnt;	
	int n;
	
	checkErr( Ivi_LockSession (vi, VI_NULL) );

	if( !Ivi_Simulating(vi) ){                /* call only when locked */
	 	ViSession   io = Ivi_IOSession(vi); /* call only when locked */
	    if( serial_flag ){
        	viCheckErr( viPrintf(io, "*IDN?\n") );
           	Delay (.05);
        }
        else
        	viCheckErr( viPrintf(io, "*IDN?") );
    
	    viCheckErr( viScanf(io, "%*[^,],%*[^,],%256[^,],%*[^\n]\n", serial_number) );  
			
	}
	checkErr( CIC_RP50_CheckStatus(vi) );                                      
	Delay(.03); 

Error:
    Ivi_UnlockSession( vi, VI_NULL );
	return error;
}							   

/*****************************************************************************
 * Function: CIC_RP50_reset                                                         
 * Purpose:  This function resets the instrument.                          
 *****************************************************************************/
ViStatus _VI_FUNC CIC_RP50_reset (ViSession vi)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Ivi_LockSession (vi, VI_NULL));

	if (!Ivi_Simulating(vi))                /* call only when locked */
	    {
	    ViSession   io = Ivi_IOSession(vi); /* call only when locked */
	    
	    checkErr( Ivi_SetNeedToCheckStatus (vi, VI_TRUE));
	    if(serial_flag)
	      {
	       viCheckErr( viPrintf (io, "SYST:REM\n"));
	       Delay (0.03);
	       viCheckErr( viPrintf (io, "*CLS\n")); //Added to avoid memory lost for I/O firmware 0.1
	       Delay (0.03);
	       viCheckErr( viPrintf (io, "SYST:REM\n"));
	       Delay (0.03);
	       viCheckErr( viPrintf (io, "*RST\n"));
	      }
	    else{
	       viCheckErr( viPrintf (io, "SYST:REM"));
	       Delay (0.03);
	       viCheckErr( viPrintf (io, "*CLS"));	//Added to avoid memory lost for I/O firmware 0.1
	       Delay (0.03);
	       viCheckErr( viPrintf (io, "SYST:REM"));
	       Delay (0.03);
	       viCheckErr( viPrintf (io, "*RST")); 
		}
		
		Delay (.03);    
		}
	
	checkErr( CIC_RP50_DefaultInstrSetup (vi));                                
	
	checkErr( CIC_RP50_CheckStatus (vi));                                      
	Delay (.03); 
Error:
    Ivi_UnlockSession (vi, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: CIC_RP50_error_query                                                     
 * Purpose:  This function queries the instrument error queue and returns   
 *           the result.                                                     
 *****************************************************************************/
ViStatus _VI_FUNC CIC_RP50_error_query (ViSession vi, ViInt32 *errCode, 
                                        ViChar errMessage[])
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Ivi_LockSession (vi, VI_NULL));
    
    if (errCode == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for Error Code");
    if (errMessage == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 3, "Null address for Error Message");

	if (!Ivi_Simulating(vi))                /* call only when locked */
	    {
	    ViSession   io = Ivi_IOSession(vi); /* call only when locked */
	
	    checkErr( Ivi_SetNeedToCheckStatus (vi, VI_TRUE));
	    if (serial_flag)
	      {
	       viCheckErr( viPrintf(io, ":SYST:ERR?\n"));
	       viCheckErr( viScanf (io, "%ld,\"%256[^\"]\n", errCode, errMessage)); 
	      }
	    else
	      {
	       viCheckErr( viPrintf(io, ":SYST:ERR?"));
	       viCheckErr( viScanf (io, "%ld,\"%256[^\"]", errCode, errMessage)); 
	      }
		 
	    }
	else if (Ivi_UseSpecificSimulation(vi)) /* call only when locked */
	    {
	        /* Simulate Error Query */
	    *errCode = 0;
	    strcpy (errMessage, "No error.");
	    }

Error:
    Ivi_UnlockSession(vi, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: CIC_RP50_StatusRegister                                                    
 * Purpose:  This function queries the Status Byte register which contains the
 *			 status summary and the Output Queue MAV bit.
 *****************************************************************************/
ViStatus _VI_FUNC CIC_RP50_StatusRegister (ViSession vi,
                                           ViInt32 *statusByte)

{
	ViStatus	error = VI_SUCCESS;
    checkErr( Ivi_LockSession (vi, VI_NULL));
    
    if (statusByte == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for Error Code");
    
	if (!Ivi_Simulating(vi))                /* call only when locked */
	    {
	    ViSession   io = Ivi_IOSession(vi); /* call only when locked */
	
	    checkErr( Ivi_SetNeedToCheckStatus (vi, VI_TRUE));
	    if (serial_flag)
	      {
	       viCheckErr( viPrintf(io, "*STB?\n"));
	       viCheckErr( viScanf (io, "%ld", statusByte)); 
	      }
	    else
	      {
	       viCheckErr( viPrintf(io, "*STB?"));
	       viCheckErr( viScanf (io, "%ld", statusByte)); 
          }	    
	    }
	else if (Ivi_UseSpecificSimulation(vi)) /* call only when locked */
	    {
	        /* Simulate Error Query */
	    *statusByte = 0;
	    }

Error:
    Ivi_UnlockSession(vi, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: CIC_RP50_error_message                                                  
 * Purpose:  This function translates the error codes returned by this       
 *           instrument driver into user-readable strings.  
 *
 *           Note:  The caller can pass VI_NULL for the vi parameter.  This 
 *           is useful if one of the init functions fail.
 *****************************************************************************/
ViStatus _VI_FUNC CIC_RP50_error_message (ViSession vi, ViStatus errorCode,
                                          ViChar errorMessage[256])
{
    ViStatus    error = VI_SUCCESS;
    
    static      IviStringValueTable errorTable = 
        {
			IVIPOWER_ERROR_CODES_AND_MSGS,
            {VI_NULL,                               VI_NULL}
        };
        
    if (vi)
        Ivi_LockSession(vi, VI_NULL);

        /* all VISA and IVI error codes are handled as well as codes in the table */
    if (errorMessage == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 3, "Null address for Error Message");

    checkErr( Ivi_GetSpecificDriverStatusDesc(vi, errorCode, errorMessage, errorTable));

Error:
    if (vi)
        Ivi_UnlockSession(vi, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: CIC_RP50_revision_query                                                  
 * Purpose:  This function returns the driver and instrument revisions.      
 *****************************************************************************/
ViStatus _VI_FUNC CIC_RP50_revision_query (ViSession vi, ViChar driverRev[], 
                                           ViChar instrRev[])
{
    ViChar      rdBuffer[BUFFER_SIZE];
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Ivi_LockSession (vi, VI_NULL));

    if (driverRev == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for Driver Revision");
    if (instrRev == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 3, "Null address for Instrument Revision");

    checkErr( Ivi_GetAttributeViString (vi, VI_NULL, IVI_ATTR_DRIVER_REVISION, 
                                        0, -1, driverRev));

	if (!Ivi_Simulating(vi))                /* call only when locked */
	    {
	    ViSession   io = Ivi_IOSession(vi); /* call only when locked */
	
	    checkErr( Ivi_SetNeedToCheckStatus (vi, VI_TRUE));
	    if(serial_flag)
	      {
	       viCheckErr( viPrintf (io, "*IDN?\n"));
	       viCheckErr( viScanf (io, "%*[^,],%*[^,],%*[^,],%256[^\n]\n", instrRev));  
	      }
	    else
	      {
	       viCheckErr( viPrintf (io, "*IDN?")); 
           viCheckErr( viScanf (io, "%*[^,],%*[^,],%*[^,],%256[^\n]", instrRev));  	
          }		
	    }
	else if (Ivi_UseSpecificSimulation(vi)) /* call only when locked */
	    {
	        /* Simulate Instrument Revision Query */
	    strcpy (instrRev, "No revision information available while simulating.");
	    }

    checkErr( CIC_RP50_CheckStatus (vi));

Error:    
    Ivi_UnlockSession(vi, VI_NULL);
    return error;
}

/*****************************************************************************
 *  Function: CIC_RP50_RemoteOnOff
 *  Purpose:  This function turns the remote light ON or OFF.
 *****************************************************************************/
ViStatus _VI_FUNC CIC_RP50_RemoteOnOff (ViSession vi,
                                        ViConstString channel,
                                        ViInt32 remoteONOFF)

{
	ViStatus	error = VI_SUCCESS;
	checkErr( Ivi_LockSession (vi, VI_NULL));

    viCheckParm (Ivi_SetAttributeViInt32 (vi, channel,
                                          CIC_RP50_ATTR_REMOTE_ON_OFF, 
                                             0, remoteONOFF),
                                             3, "Remote On or Off");
    
    Delay (.03);  
    checkErr( CIC_RP50_CheckStatus (vi));

Error:
    Ivi_UnlockSession (vi, VI_NULL);
	return error;
}


/*****************************************************************************
 *  Function: CIC_RP50_DisplayMode
 *  Purpose:  This function sets the display mode to frequency or current
 *			  limit/measure mode.
 *****************************************************************************/
ViStatus _VI_FUNC CIC_RP50_DisplayMode (ViSession vi, ViConstString channel,
                                        ViInt32 displayMode)
{
	ViStatus	error = VI_SUCCESS;
	checkErr( Ivi_LockSession (vi, VI_NULL));
	
	if( model_number != 2001 || model_number != 2003 ){
	  	if( displayMode > 5 ){
          	return CIC_RP50_ERROR_NOT_SUPPORTED;  
       	} 
       	else{
          	viCheckParm( Ivi_SetAttributeViInt32 (vi, channel,
                                                CIC_RP50_ATTR_DISPLAY_MODE, 
                                                0, displayMode),
                                                3, "Display On or Off" );
      	}   
   	}
    else{
       	viCheckParm( Ivi_SetAttributeViInt32 (vi, channel,
                                             CIC_RP50_ATTR_DISPLAY_MODE, 
                                             0, displayMode),
                                             3, "Display On or Off" );
  	}
    checkErr( CIC_RP50_CheckStatus(vi) );

Error:
    Ivi_UnlockSession( vi, VI_NULL );
	return error;
}

/*****************************************************************************
 *  Function: CIC_RP50_DisplayModeQuery
 *  Purpose:  This function queries the display mode.
 *****************************************************************************/
ViStatus _VI_FUNC CIC_RP50_DisplayModeQuery (ViSession vi,
                                             ViConstString channel,
                                             ViInt32 *displayStatus)
{
	ViStatus	error = VI_SUCCESS;
	checkErr( Ivi_LockSession (vi, VI_NULL));

    
	if (!Ivi_Simulating(vi))                /* call only when locked */
	    {
	    ViSession   io = Ivi_IOSession(vi); /* call only when locked */
	    
	    checkErr( Ivi_SetNeedToCheckStatus (vi, VI_TRUE));
	    if (serial_flag)
	      {
	       viCheckErr( viPrintf (io, "DISP:MODE?\n"));
	       Delay (.05);
	       viCheckErr( viScanf (io, "%d\n", displayStatus)); 
	       Delay (.05);  
	      }
	    else
	      {
	       viCheckErr( viPrintf (io, "DISP:MODE?")); 
	       viCheckErr( viScanf (io, "%d", displayStatus)); 
          }	    
	    }
	
	checkErr( CIC_RP50_DefaultInstrSetup (vi));                                
	
    checkErr( CIC_RP50_CheckStatus (vi));
	Delay (.05); 
Error:
    Ivi_UnlockSession (vi, VI_NULL);
	return error;
}

/*****************************************************************************
 *  Function: CIC_RP50_ConnectOutput
 *  Purpose:  This function connects a channel to its output.
 *****************************************************************************/
ViStatus _VI_FUNC CIC_RP50_ConnectOutput (ViSession vi, ViConstString channel)
{
    ViStatus  error = VI_SUCCESS;

    checkErr( Ivi_LockSession (vi, VI_NULL));

    checkErr( Ivi_SetAttributeViBoolean (vi, channel,
                                         CIC_RP50_ATTR_OUTPUT_CONNECTED,
                                         0, VI_TRUE));
    
    checkErr( CIC_RP50_CheckStatus (vi));

Error:
    Ivi_UnlockSession (vi, VI_NULL);
    return error;
}

/*****************************************************************************
 *  Function: CIC_RP50_DisconnectOutput
 *  Purpose:  This function disconnects a channel from its output.
 *****************************************************************************/
ViStatus _VI_FUNC CIC_RP50_DisconnectOutput (ViSession vi, ViConstString channel)
{
    ViStatus error = VI_SUCCESS;

    checkErr( Ivi_LockSession (vi, VI_NULL));

    checkErr( Ivi_SetAttributeViBoolean (vi, channel,
                                         CIC_RP50_ATTR_OUTPUT_CONNECTED,
                                         0, VI_FALSE));
    
    checkErr( CIC_RP50_CheckStatus (vi));

Error:
    Ivi_UnlockSession (vi, VI_NULL);
    return error;
}

/*****************************************************************************
 *  Function: CIC_RP50_DropCommands
 *  Purpose:  This function controls the output drop to zero voltage only. 
 *            This function can also be used to drop the output to zero with a
 *            phase angle of 0, 90, 180, 270 degree or at random phase angle.
 *****************************************************************************/
/*ViStatus _VI_FUNC CIC_RP50_DropCommands (ViSession vi,
                                         ViConstString channel,
                                         ViInt32 startphasemode,//at certain phase of zero phase
                                         ViReal64 phaseAngle)//specified phase angle
{
	ViStatus	error = VI_SUCCESS;

    checkErr( Ivi_LockSession (vi, VI_NULL));

	if (!Ivi_Simulating(vi)){                // call only when locked 
		ViSession   io = Ivi_IOSession(vi); // call only when locked 
	    
	    checkErr( Ivi_SetNeedToCheckStatus (vi, VI_TRUE));
	    
        if (model_number != 2001){
          	return CIC_RP50_ERROR_NOT_SUPPORTED;     
        }
        else{
		 	CIC_RP50_ConnectOutput (vi, channel); 
	    
	  		if (startphasemode){
		    	if( phaseAngle >= 0.0 && phaseAngle < 90.0 ){
	            	phaseAngle = 0.0;
				}
				else if( phaseAngle >= 90.0 && phaseAngle < 180.0 ){
	             	phaseAngle = 90.0;
				}
				else if( phaseAngle >= 180.0 && phaseAngle < 270.0 ){
	             	phaseAngle = 180.0;
				}
				else if( phaseAngle >= 270.0 && phaseAngle < 360.0 ){
	             	phaseAngle = 270.0;
				}
		        if (serial_flag){
	             	viCheckErr( viPrintf (io, "OUTP:STAR:STAT 1\n"));
	             	Delay (.05);
	             	viCheckErr( viPrintf (io, "OUTP:STAR:PHAS %f\n", phaseAngle));
	             	Delay (.05);
	             	viCheckErr( viPrintf (io, "OUTP:DROP\n"));     
	             	Delay (.05);
	          	}
		        else{
		        	viCheckErr( viPrintf (io, "OUTP:STAR:STAT 1"));
		            Delay (.03);    
		            viCheckErr( viPrintf (io, "OUTP:STAR:PHAS %f", phaseAngle));
		            Delay (.03);    
		            viCheckErr( viPrintf (io, "OUTP:DROP"));     
		            Delay (.03);      
		       	}	    
		 	}
		    else{
		    	if (serial_flag){
		            viCheckErr( viPrintf (io, "OUTP:STAR:STAT 0\n")); 
		            Delay (.05); 
		            viCheckErr( viPrintf (io, "OUTP:DROP\n"));     
		            Delay (.05);
		        }
		        else{
		            viCheckErr( viPrintf (io, "OUTP:STAR:STAT 0"));   
		            Delay (.03);    
		            viCheckErr( viPrintf (io, "OUTP:DROP"));     
				 	Delay (.03);    
		        }	    
	      	}
      	}
	}
	
	checkErr( CIC_RP50_DefaultInstrSetup (vi));                                
	
    checkErr( CIC_RP50_CheckStatus (vi));


Error:
	return error;
}*/

/*****************************************************************************
 *  Function: CIC_RP50_DropCommands
 *  Purpose:  This function controls the output drop logic for the 2001 RP. 
 *            This function can be phase angle and time duration controlled.
 *****************************************************************************/
/*ViStatus _VI_FUNC CIC_RP50_DropCommands (ViSession vi,
                                         ViConstString channel,
                                         ViInt32 startphasemode,
                                         ViReal64 phaseAngle,
                                         ViReal64 outputDrop, ViReal64 voltage,
                                         ViInt16 dropMode)

{
	ViStatus	error = VI_SUCCESS;

    checkErr( Ivi_LockSession (vi, VI_NULL));

	if (!Ivi_Simulating(vi))                // call only when locked 
	   {
	    ViSession   io = Ivi_IOSession(vi); // call only when locked 
	    
	    checkErr( Ivi_SetNeedToCheckStatus (vi, VI_TRUE));
	    
        if (model_number != 2001)
          {
           return CIC_RP50_ERROR_NOT_SUPPORTED;     
          }
        else
          {
           CIC_RP50_GenerateACVoltage (vi, channel, 0.0);
		   CIC_RP50_ConnectOutput (vi, channel); 
	    
           if (startphasemode)
	         {
	          if (serial_flag)
	            {
	             viCheckErr( viPrintf (io, "OUTP:STAR:STAT 1\n"));
	             Delay (.05);
	             viCheckErr( viPrintf (io, "OUTP:STAR:PHAS %f\n", phaseAngle));
	             Delay (.05);
	             if (dropMode == 0) 
	               {
	                viCheckErr( viPrintf (io, "OUTP:DROP\n"));     
	               }
	             else
	               {
	                viCheckErr( viPrintf (io, "OUTP:DROP %f\n", outputDrop));
	               }
	             Delay (.05);
	             viCheckErr( viPrintf (io, "VOLT %f\n", voltage));
	             Delay (.05);
	            }
	          else
	            {
	             viCheckErr( viPrintf (io, "OUTP:STAR:STAT 1"));
	             Delay (.03);    
	             viCheckErr( viPrintf (io, "OUTP:STAR:PHAS %f", phaseAngle));
	             Delay (.03);    
	             if (dropMode == 0)     
	               {
	                viCheckErr( viPrintf (io, "OUTP:DROP"));     
	               }
	             else
	               {
	                viCheckErr( viPrintf (io, "OUTP:DROP %f", outputDrop));
	               }
	             Delay (.03);      
	             viCheckErr( viPrintf (io, "VOLT %f", voltage));
	             Delay (.03);    
	            }	    
	         }
	       else
	         {
	          if (serial_flag)
	            {
	             viCheckErr( viPrintf (io, "OUTP:STAR:STAT 0\n")); 
	             Delay (.05); 
	             if (dropMode == 0)    
	               {
	                viCheckErr( viPrintf (io, "OUTP:DROP\n"));     
	               }
	             else
	               {
	                viCheckErr( viPrintf (io, "OUTP:DROP %f\n", outputDrop));
	               }
	             Delay (.05);
	             viCheckErr( viPrintf (io, "VOLT %f\n", voltage));
	             Delay (.05);
	            }
	          else
	            {
	             viCheckErr( viPrintf (io, "OUTP:STAR:STAT 0"));   
	             Delay (.03);    
	             if (dropMode == 0)    
	               {
	                viCheckErr( viPrintf (io, "OUTP:DROP"));     
	               }
	             else
	               {
	                viCheckErr( viPrintf (io, "OUTP:DROP %f", outputDrop));
	               }

				 Delay (.03);    
	             viCheckErr( viPrintf (io, "VOLT %f", voltage));
	            }	    
             }
          }
	   }
	
	checkErr( CIC_RP50_DefaultInstrSetup (vi));                                
	
    checkErr( CIC_RP50_CheckStatus (vi));


Error:
	return error;
}*/

/*****************************************************************************
 *  Function: CIC_RP50_DropSettingQuery
 *  Purpose:  This function queries the output drop logic for the 2001 RP. 
 *            Only the phase angle status (ON or OFF) and the programmed phase
 *            angles (0, 90, 180, 270) can be queried.
 *****************************************************************************/
/*ViStatus _VI_FUNC CIC_RP50_DropSettingQuery (ViSession vi,
                                             ViConstString channelName,
                                             ViInt32 *phaseAngleStatus,
                                             ViInt32 *programmedPhaseAngle)
{
	ViStatus	error = VI_SUCCESS;

	if (!Ivi_Simulating(vi))                // call only when locked 
	   {
	    ViSession   io = Ivi_IOSession(vi); // call only when locked 
	    
	    checkErr( Ivi_SetNeedToCheckStatus (vi, VI_TRUE));
	    if (model_number != 2001)
	      {
	       return CIC_RP50_ERROR_NOT_SUPPORTED;     
          }
        else
          {
           if (serial_flag)
	         {
	          viCheckErr( viPrintf (io, "OUTP:STAR?\n"));
	          Delay (.02);
	          viCheckErr( viScanf (io, "%d\n", phaseAngleStatus)); 
	          Delay (.02);  
	          viCheckErr( viPrintf (io, "OUTP:STAR:PHAS?\n"));
	          Delay (.02);
	          viCheckErr( viScanf (io, "%d\n", programmedPhaseAngle)); 
	          Delay (.02);  
	         }
	       else
	         {
	          viCheckErr( viPrintf (io, "OUTP:STAR?"));
	          viCheckErr( viScanf (io, "%d", phaseAngleStatus)); 
	          viCheckErr( viPrintf (io, "OUTP:STAR:PHAS?"));
	          viCheckErr( viScanf (io, "%d", programmedPhaseAngle)); 
	         }	    
	       }
		}
	checkErr( CIC_RP50_DefaultInstrSetup (vi));                                
	
    checkErr( CIC_RP50_CheckStatus (vi));

Error:
	return error;
}  */

/*****************************************************************************
 *  Function: CIC_RP50_PhaseDropCommands
 *  Purpose:  This function controls the output drop logic for the 2003 RP. 
 *            This function can be phase angle and time duration controlled.
 *****************************************************************************/
ViStatus _VI_FUNC CIC_RP50_PhaseDropCommands (ViSession vi,
                                         ViConstString channel,
                                         ViInt32 startphasemode,
                                         ViReal64 phaseAngle,
                                         ViReal64 outputDrop,
                                         ViInt16 dropMode,
                                         ViInt16 selectPhase)
{
	ViStatus	error = VI_SUCCESS;
    checkErr( Ivi_LockSession( vi, VI_NULL) );

	if( !Ivi_Simulating(vi) ){                /* call only when locked */
	   
	    ViSession   io = Ivi_IOSession(vi); /* call only when locked */
	    checkErr( Ivi_SetNeedToCheckStatus(vi, VI_TRUE) );
	    
        if( model_number == 2001 || model_number == 2003 ){
	        if( phaseAngle == 0 || phaseAngle == 90 || phaseAngle == 180 || phaseAngle == 270 ){
	           	if( selectPhase == 0 ){
	           		if( model_number == 2003 ){	
	           			if( serial_flag ){
		             		viCheckErr( viPrintf(io, "INST:COUP 1\n") );
		            	}
		            	else  	
		             		viCheckErr( viPrintf(io, "INST:COUP 1") );
	             		
		             	Delay (.03);    	
					}
		        }    
	           	else if( selectPhase == 1 ){
	           		if( serial_flag ){
		             	viCheckErr( viPrintf(io, "INST:COUP 0\n") );
		            	Delay(.02);
		            	viCheckErr( viPrintf(io, "INST:NSEL 1\n") );
		            	Delay(.02);
		            }
		            else{  	
		             	viCheckErr( viPrintf(io, "INST:COUP 0") );
		        		Delay(.02);
		        		viCheckErr( viPrintf(io, "INST:NSEL 1") );
						Delay(.02);
					}
		        }
		        else if( selectPhase == 2 ){
	           		if( serial_flag ){
		             	viCheckErr( viPrintf(io, "INST:COUP 0\n") );
		            	viCheckErr( viPrintf(io, "INST:NSEL 2\n") );
		            	Delay(.02);
		            }
		            else{
		             	viCheckErr( viPrintf(io, "INST:COUP 0") );
		             	Delay (.03);    
		        		viCheckErr( viPrintf(io, "INST:NSEL 2") );
		        		Delay(.02);
		        	}
		        }
	           	else if( selectPhase == 3 ){
	           		if( serial_flag ){
		             	viCheckErr( viPrintf(io, "INST:COUP 0\n") );
		             	Delay (.03);    
		            	viCheckErr( viPrintf(io, "INST:NSEL 3\n") );
		            	Delay(.02);
		            }
		            else{  	
		             	viCheckErr( viPrintf(io, "INST:COUP 0") );
		             	Delay (.03);    
		        		viCheckErr( viPrintf(io, "INST:NSEL 3") );
						Delay(.02);
					}
		        }
	           	
	           	if( startphasemode ){   
		          	if( serial_flag ){
		             	viCheckErr( viPrintf (io, "OUTP:STAR:STAT 1\n") );
		             	Delay (.02);
		             	viCheckErr( viPrintf (io, "OUTP:STAR:PHAS %f\n", phaseAngle) );
		             	Delay (.02);
	           	
		           		if( dropMode == 0 ){
		                	viCheckErr( viPrintf (io, "OUTP:DROP\n") );     
			            }
			            else{
		    	            viCheckErr( viPrintf (io, "OUTP:DROP %f\n", outputDrop) );
		        	   	}
		        	   	Delay (.03);    
		            }
		        	else{
		            	viCheckErr( viPrintf (io, "OUTP:STAR:STAT 1") );
		            	Delay (.03);    
		            	viCheckErr( viPrintf (io, "OUTP:STAR:PHAS %f", phaseAngle) );
		            	Delay (.03);    
	            	
		            	if( dropMode == 0 ){
		              		viCheckErr( viPrintf (io, "OUTP:DROP") );     
		           		}
		            	else{
		                	viCheckErr( viPrintf(io, "OUTP:DROP %f", outputDrop) );
		            	}
		            	Delay (.03);    
		    		}
		  		}
		    	else{
		      		if( serial_flag ){
		           		viCheckErr( viPrintf (io, "OUTP:STAR:STAT 0\n") ); 
		            	Delay (.02); 
		            	viCheckErr( viPrintf (io, "OUTP:STAR:PHAS %f\n", phaseAngle) );
		             	Delay (.02);
	            	
		            	if( dropMode == 0 ){
		              		viCheckErr( viPrintf (io, "OUTP:DROP\n") );     
		           		}
		            	else{
		            		viCheckErr( viPrintf (io, "OUTP:DROP %f\n", outputDrop) );
		            	}
		            	Delay (.03);    
		            }
		        	else{
		           		viCheckErr( viPrintf (io, "OUTP:STAR:STAT 0"));   
		            	Delay(.02);
		            	viCheckErr( viPrintf (io, "OUTP:STAR:PHAS %f", phaseAngle) );
		            	Delay(.02);
	            	
		            	if( dropMode == 0 ){
		              		viCheckErr( viPrintf (io, "OUTP:DROP"));     
		          		}
		      			else{
		                	viCheckErr( viPrintf (io, "OUTP:DROP %f", outputDrop));
		        		}
		        		Delay (.03);    
		  			}	
	     		}
	  		}
	  		else
	  			MessagePopup ("Error Message", "\nOutput Drop function only supports 0, 90, 180, 270 phase angle");    
	  	}
        else{
			return CIC_RP50_ERROR_NOT_SUPPORTED;     
		}
	}
	checkErr( CIC_RP50_DefaultInstrSetup(vi) );                                
    checkErr( CIC_RP50_CheckStatus(vi) );

Error:
	return error;
}

/*****************************************************************************
 *  Function: CIC_RP50_PhaseDropSettingQuery
 *  Purpose:  This function queries the output drop logic for the 2001 RP. 
 *            Only the phase angle status (ON or OFF) and the programmed phase
 *            angles (0, 90, 180, 270) can be queried.
 *****************************************************************************/
ViStatus _VI_FUNC CIC_RP50_PhaseDropSettingQuery (ViSession vi,
                                             ViConstString channelName,
                                             ViInt16 selectPhase,
                                             ViInt32 *phaseAngleStatus,
                                             ViInt32 *programmedPhaseAngle)
{
	ViStatus	error = VI_SUCCESS;

	if( !Ivi_Simulating(vi) ){                /* call only when locked */
	   
	    ViSession   io = Ivi_IOSession(vi); /* call only when locked */
	    checkErr( Ivi_SetNeedToCheckStatus (vi, VI_TRUE) );
	    
	    if( model_number != 2001 || model_number != 2003 ){
	       	return CIC_RP50_ERROR_NOT_SUPPORTED;     
        }
        else{
           	if( selectPhase == 0 ){
           		if( model_number == 2003 ){	
           			if( serial_flag ){
	             		viCheckErr( viPrintf(io, "INST:COUP 1\n") );
	            	}
	            	else  	
	             		viCheckErr( viPrintf(io, "INST:COUP 1") );
					
					Delay(.03);  
				}
	        }    
           	else if( selectPhase == 1 ){
           		if( serial_flag ){
	            	viCheckErr( viPrintf(io, "INST:NSEL 1\n") );
	            }
	            else  	
	        		viCheckErr( viPrintf(io, "INST:NSEL 1") );
	        		
	        	Delay(.03);  	
			}
	        else if( selectPhase == 2 ){
           		if( serial_flag ){
	            	viCheckErr( viPrintf(io, "INST:NSEL 2\n") );
	            }
	            else
	        		viCheckErr( viPrintf(io, "INST:NSEL 2") );
	        		
	        	Delay(.03);  	
	        }
           	else if( selectPhase == 3 ){
           		if( serial_flag ){
	            	viCheckErr( viPrintf(io, "INST:NSEL 3\n") );
	            }
	            else  	
	        		viCheckErr( viPrintf(io, "INST:NSEL 3") );
	        		
	        	Delay(.03);  	
			}	
           	
           	if( serial_flag ){
	          	viCheckErr( viPrintf (io, "OUTP:STAR?\n"));
	          	Delay (.02);
	          	viCheckErr( viScanf (io, "%d\n", phaseAngleStatus)); 
	          	Delay (.02);  
	          	viCheckErr( viPrintf (io, "OUTP:STAR:PHAS?\n"));
	          	Delay (.02);
	          	viCheckErr( viScanf (io, "%d\n", programmedPhaseAngle)); 
	          	Delay (.02);  
	       	}
	       	else{
	          	viCheckErr( viPrintf (io, "OUTP:STAR?") );
	          	viCheckErr( viScanf (io, "%d", phaseAngleStatus) ); 
	          	viCheckErr( viPrintf (io, "OUTP:STAR:PHAS?") );
	          	viCheckErr( viScanf (io, "%d", programmedPhaseAngle) ); 
	       	}	    
	       	Delay(.03);  
	   	}
	}
	checkErr( CIC_RP50_DefaultInstrSetup(vi) );                                
    checkErr( CIC_RP50_CheckStatus(vi) );

Error:
	return error;
}

/*****************************************************************************
 *  Function: CIC_RP50_ConfigureLimits
 *  Purpose:  This function configures the over-voltage and over-current
 *            protection.
 *****************************************************************************/
ViStatus _VI_FUNC CIC_RP50_ConfigureLimits (ViSession vi,
                                            ViConstString channelName,
                                            ViReal64 *frequencyLowLimit,
                                            ViReal64 *frequencyHighLimit,
                                            ViReal64 *currentLimit,
                                            ViReal64 *voltageLimit,
                                            ViInt32 *modelNumber)
{
    ViStatus error = VI_SUCCESS;

    checkErr( Ivi_LockSession (vi, VI_NULL));

    /* Get limit from the IviInit */
    
    *frequencyLowLimit = freq_low_limit;
    *frequencyHighLimit = freq_high_limit;
    *voltageLimit = volt_high_limit;
    *currentLimit = curr_high_limit;
    *modelNumber= model_number;
     
    checkErr( CIC_RP50_CheckStatus (vi));

Error:
    Ivi_UnlockSession (vi, VI_NULL);
    return error;
}

/*****************************************************************************
 *  Function: CIC_RP50_GenerateACVoltageRange
 *  Purpose:  This function configures the power supply to generate an
 *            AC voltage range on a channel and initiates the output change.
 *            
 *****************************************************************************/
ViStatus _VI_FUNC CIC_RP50_GenerateACVoltageRange (ViSession vi,
                                                   ViConstString channel,
                                                   ViReal64 voltageRange)
{
	ViStatus	error = VI_SUCCESS;
	checkWarn( Ivi_LockSession (vi, VI_NULL));

    viCheckParm( Ivi_SetAttributeViReal64 (vi, channel,
    									   CIC_RP50_ATTR_VOLTAGE_RANGE,
    									   0, voltageRange),
    									   2, "Voltage Range");
    
    checkWarn( CIC_RP50_CheckStatus (vi));
    

Error:
    Ivi_UnlockSession (vi, VI_NULL);
	return error;
}

/*****************************************************************************
 *  Function: CIC_RP50_GenerateACVoltage
 *  Purpose:  This function configures the power supply to generate an
 *            AC voltage on a channel and initiates the output change.
 *            
 *****************************************************************************/
ViStatus _VI_FUNC CIC_RP50_GenerateACVoltage (ViSession vi, ViConstString channel,
                                              ViReal64 amplitude)
{
 	ViStatus error = VI_SUCCESS;
	ViReal64 voltrange;
	
    checkWarn( Ivi_LockSession (vi, VI_NULL));
    
    viCheckParm( Ivi_GetAttributeViReal64 (vi, channel,
    									   CIC_RP50_ATTR_VOLTAGE_RANGE,
    									   0, &voltrange),
    									   2, "Voltage Range");
    									   
	if (voltrange == 1.0)
	  	voltrange = volt_high_limit / 2;
	else if (voltrange == 2.0)
	  	voltrange = volt_high_limit;
	  
    if ((amplitude > voltrange) || (amplitude < 0))
        return CIC_RP50_ERROR_OVER_VOLTAGE ;
    viCheckParm( Ivi_SetAttributeViReal64 (vi, channel,
    									   CIC_RP50_ATTR_ACV_AMPLITUDE,
    									   0, amplitude), 
    									   4, "Amplitude");  

    checkWarn( CIC_RP50_CheckStatus (vi));
    
	Error:
    
    Ivi_UnlockSession (vi, VI_NULL);
    return error;
}

/*****************************************************************************
 *  Function: CIC_RP50_GenerateACCurrent
 *  Purpose:  This function configures the power supply to generate an
 *            AC current on a channel and initiates the output change.
 *            
 *****************************************************************************/
ViStatus _VI_FUNC CIC_RP50_GenerateACCurrent (ViSession vi, ViConstString channel,
                                              ViReal64 currentLimit)
{
    ViStatus error = VI_SUCCESS;
	ViReal64 currentrange, voltrange;
	
    checkWarn( Ivi_LockSession (vi, VI_NULL));

    viCheckParm( Ivi_GetAttributeViReal64 (vi, channel,
    									   CIC_RP50_ATTR_VOLTAGE_RANGE,
    									   0, &voltrange),
       									   2, "Voltage Range"); 
    
    if (voltrange == 1.0)
	  voltrange = volt_high_limit / 2;
	else if (voltrange == 2.0)
	  voltrange = volt_high_limit;
	  
	if (voltrange > (volt_high_limit/2))
       currentrange = curr_high_limit /2;
    else
       currentrange = curr_high_limit;
       
    if ((currentLimit > currentrange) || (currentLimit < 0))
       return CIC_RP50_ERROR_OVER_CURRENT;
      
    viCheckParm( Ivi_SetAttributeViReal64 (vi, channel,
    									   CIC_RP50_ATTR_ACC_AMPLITUDE,
    									   0, currentLimit),
    									   5, "Current Limit");
    
    checkWarn( CIC_RP50_CheckStatus (vi));

Error:
    Ivi_UnlockSession (vi, VI_NULL);
    return error;
}

/*****************************************************************************
 *  Function: CIC_RP50_GenerateACFrequency
 *  Purpose:  This function configures the power supply to generate an
 *            AC frequency on a channel and initiates the output change.
 *           
 *****************************************************************************/
ViStatus _VI_FUNC CIC_RP50_GenerateACFrequency (ViSession vi,
                                                ViConstString channel,
                                                ViReal64 frequency)
{
	ViStatus error = VI_SUCCESS;

    checkWarn( Ivi_LockSession (vi, VI_NULL));

    if ((frequency < freq_low_limit) || (frequency > freq_high_limit))
       return  CIC_RP50_ERROR_OVER_FREQUENCY;
       
    viCheckParm( Ivi_SetAttributeViReal64 (vi, channel,
    									   CIC_RP50_ATTR_FREQUENCY,
    									   0, frequency),
    									   4, "Frequency");
    
    checkWarn( CIC_RP50_CheckStatus (vi));  
    

Error:
    Ivi_UnlockSession (vi, VI_NULL);
    return error;
}

/*****************************************************************************
 *  Function: CIC_RP50_MeasureOutput
 *  Purpose:  This function measures the output of a channel.
 *****************************************************************************/
ViStatus _VI_FUNC CIC_RP50_MeasureOutput (ViSession vi, ViConstString channel,
                                          ViInt32 measureFunction,
                                          ViReal64 *measurement)
{
    ViStatus error = VI_SUCCESS;

    checkErr( Ivi_LockSession (vi, VI_NULL));

    if (measurement == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 5, "Null address for Measurement");
        
    if (measureFunction == CIC_RP50_VAL_AC_VOLTS) 
      {
        ViSession     io = Ivi_IOSession(vi); // call only when locked
        if (serial_flag)
          {
           viCheckErr( viPrintf (io, "MEAS:VOLT?\n")); 
           viCheckErr( viScanf (io, "%lf\n", measurement));  
          }
        else
          {
           viCheckErr( viPrintf (io, "MEAS:VOLT?"));   
           viCheckErr( viScanf (io, "%lf", measurement));  
          }          
      }
    else if (measureFunction == CIC_RP50_VAL_AC_CURRENT) 
      {
        ViSession     io = Ivi_IOSession(vi); // call only when locked
        if (serial_flag)
          {
           viCheckErr( viPrintf (io, "MEAS:CURR?\n")); 
           viCheckErr( viScanf (io, "%lf\n", measurement));  
          }
        else
          {
           viCheckErr( viPrintf (io, "MEAS:CURR?"));   
           viCheckErr( viScanf (io, "%lf", measurement));  
          }          
      }
    else if (measureFunction == CIC_RP50_VAL_APP_POWER)
      {
        ViSession     io = Ivi_IOSession(vi); // call only when locked       
        if (model_number != 2001)
          {
           ViReal64 ACVOLT, ACCURR, ACPOWER;
        
           if (serial_flag)
             {
              viCheckErr( viPrintf (io, "MEAS:VOLT?\n")); 
              viCheckErr( viScanf (io, "%lf\n", &ACVOLT));
              viCheckErr( viPrintf (io, "MEAS:CURR?\n")); 
              viCheckErr( viScanf (io, "%lf\n", &ACCURR));  
             }
           else
             {
              viCheckErr( viPrintf (io, "MEAS:VOLT?")); 
              viCheckErr( viScanf (io, "%lf\n", &ACVOLT));
              viCheckErr( viPrintf (io, "MEAS:CURR?"));   
              viCheckErr( viScanf (io, "%lf", &ACCURR));  
             }          
          
           ACPOWER = ACVOLT * ACCURR;
           *measurement = ACPOWER;
          }
        else
          {
           if (serial_flag)
             {
              viCheckErr( viPrintf (io, "MEAS:POW:APP?\n")); 
              viCheckErr( viScanf (io, "%lf\n", measurement));  
             }
           else
             {
              viCheckErr( viPrintf (io, "MEAS:POW:APP?"));   
              viCheckErr( viScanf (io, "%lf", measurement));  
             }    
          }
      }
    else if (measureFunction == CIC_RP50_VAL_FREQUENCY) 
      {
        ViSession     io = Ivi_IOSession(vi); // call only when locked
        if (serial_flag)
          {
           viCheckErr( viPrintf (io, "FREQ?\n")); 
           viCheckErr( viScanf (io, "%lf\n", measurement));  
          }
        else
          {
           viCheckErr( viPrintf (io, "FREQ?"));   
           viCheckErr( viScanf (io, "%lf", measurement));  
          }          
       }
    else if (measureFunction == CIC_RP50_VAL_PEAK_CURRENT) 
      {
        ViSession     io = Ivi_IOSession(vi); // call only when locked
        if (model_number != 2001)
          {
           return CIC_RP50_ERROR_NOT_SUPPORTED;  
          }
        else
          {
           if (serial_flag)
             {
              viCheckErr( viPrintf (io, "MEAS:CURR:AMPL:MAX?\n")); 
              viCheckErr( viScanf (io, "%lf\n", measurement));  
             }
           else
             {
              viCheckErr( viPrintf (io, "MEAS:CURR:AMPL:MAX?"));   
              viCheckErr( viScanf (io, "%lf", measurement));  
             }
          }
       }
    else if (measureFunction == CIC_RP50_VAL_CREST_FACTOR) 
      {
        ViSession     io = Ivi_IOSession(vi); // call only when locked
        if (model_number != 2001)
          {
           return CIC_RP50_ERROR_NOT_SUPPORTED;  
          }
        else
          {
           if (serial_flag)
             {
              viCheckErr( viPrintf (io, "MEAS:CURR:CRES?\n")); 
              viCheckErr( viScanf (io, "%lf\n", measurement));  
             }
           else
             {
              viCheckErr( viPrintf (io, "MEAS:CURR:CRES?"));   
              viCheckErr( viScanf (io, "%lf", measurement));  
             }          
          }
       }
    else if (measureFunction == CIC_RP50_VAL_POWER) 
      {
        ViSession     io = Ivi_IOSession(vi); // call only when locked
        if (model_number != 2001)
          {
           return CIC_RP50_ERROR_NOT_SUPPORTED;  
          }
        else
          {
           if (serial_flag)
             {
              viCheckErr( viPrintf (io, "MEAS:POW?\n")); 
              viCheckErr( viScanf (io, "%lf\n", measurement));  
             }
           else
             {
              viCheckErr( viPrintf (io, "MEAS:POW?"));   
              viCheckErr( viScanf (io, "%lf", measurement));  
             }          
          }
       }
    else if (measureFunction == CIC_RP50_VAL_POWER_FACTOR) 
      {
        ViSession     io = Ivi_IOSession(vi); // call only when locked
        if (model_number != 2001)
          {
           return CIC_RP50_ERROR_NOT_SUPPORTED;  
          }
        else
          {
           if (serial_flag)
             {
              viCheckErr( viPrintf (io, "MEAS:POW:PFAC?\n")); 
              viCheckErr( viScanf (io, "%lf\n", measurement));  
             }
           else
             {
              viCheckErr( viPrintf (io, "MEAS:POW:PFAC?"));   
              viCheckErr( viScanf (io, "%lf", measurement));  
             }          
          }
       }
     
Error:
    Ivi_UnlockSession (vi, VI_NULL);
    return error;
}

/*****************************************************************************
 *  Function: CIC_RP50_PhaseMeasureOutput
 *  Purpose:  This function measures the output of a specified phase.
 *****************************************************************************/
ViStatus _VI_FUNC CIC_RP50_PhaseMeasureOutput (ViSession vi, ViConstString channel,
                                          ViInt32 measureFunction,
                                          ViInt16 selectPhase,
                                          ViReal64 *measurement)
{
   	ViSession   io = Ivi_IOSession(vi); /* call only when locked */
	
   	ViStatus error = VI_SUCCESS;
    checkErr( Ivi_LockSession (vi, VI_NULL) );

    if( measurement == VI_NULL )
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 5, "Null address for Measurement" );
    
    if( selectPhase == 0 ){
    	if( model_number == 2003 ){	
        	if( serial_flag ){
	        	viCheckErr( viPrintf(io, "INST:COUP 1\n") );
	       	}
	       	else  	
	        	viCheckErr( viPrintf(io, "INST:COUP 1") );
			Delay(.03);  
		}
	}    
    
    else if( selectPhase == 1 ){
    	if( serial_flag ){
	       	viCheckErr( viPrintf(io, "INST:NSEL 1\n") );
	    }
	    else  	
	    	viCheckErr( viPrintf(io, "INST:NSEL 1") );
		Delay(.03);  
	}
	
	else if( selectPhase == 2 ){
       	if( serial_flag ){
	       	viCheckErr( viPrintf(io, "INST:NSEL 2\n") );
	    }
	    else
	   		viCheckErr( viPrintf(io, "INST:NSEL 2") );
	   	Delay(.03);  
	}
    
    else if( selectPhase == 3 ){
      	if( serial_flag ){
	       	viCheckErr( viPrintf(io, "INST:NSEL 3\n") );
	    }
	    else  	
	   		viCheckErr( viPrintf(io, "INST:NSEL 3") );
	   	Delay(.03);  
	}	
    
    if( measureFunction == CIC_RP50_VAL_AC_VOLTS ){
        ViSession     io = Ivi_IOSession(vi); // call only when locked
        if( serial_flag ){
           	viCheckErr( viPrintf(io, "MEAS:VOLT?\n") ); 
           	Delay(0.02);
           	viCheckErr( viScanf(io, "%lf\n", measurement) );  
        	Delay(0.02);
        }
        else{
           	viCheckErr( viPrintf (io, "MEAS:VOLT?") );   
           	viCheckErr( viScanf (io, "%lf", measurement) );  
       	}          
   	}
    
    else if( measureFunction == CIC_RP50_VAL_AC_CURRENT ){
    	ViSession     io = Ivi_IOSession(vi); // call only when locked
        if( serial_flag ){
           	viCheckErr( viPrintf (io, "MEAS:CURR?\n")); 
           	Delay(0.02);
           	viCheckErr( viScanf (io, "%lf\n", measurement));  
      		Delay(0.02);
      	}
        else{
           	viCheckErr( viPrintf (io, "MEAS:CURR?"));   
           	viCheckErr( viScanf (io, "%lf", measurement));  
       	}          
   	}
    
    else if( measureFunction == CIC_RP50_VAL_APP_POWER ){
      	ViSession     io = Ivi_IOSession(vi); // call only when locked       
    
        if( model_number == 2001 || model_number == 2003 ){
           	if( serial_flag ){
              	viCheckErr( viPrintf(io, "MEAS:POW:APP?\n") ); 
              	Delay(0.02);
              	viCheckErr( viScanf(io, "%lf\n", measurement) );  
           		Delay(0.02);
           	}
           	else{
              	viCheckErr( viPrintf(io, "MEAS:POW:APP?") );   
              	viCheckErr( viScanf(io, "%lf", measurement) );  
           	}    
      	}
  	 	else{
  			ViReal64 ACVOLT, ACCURR, ACPOWER;
           	if( serial_flag ){
              	viCheckErr( viPrintf (io, "MEAS:VOLT?\n") ); 
              	Delay(0.02);
              	viCheckErr( viScanf (io, "%lf\n", &ACVOLT) );
              	Delay(0.02);
              	viCheckErr( viPrintf (io, "MEAS:CURR?\n") ); 
              	Delay(0.02);
              	viCheckErr( viScanf (io, "%lf\n", &ACCURR) );  
           		Delay(0.02);
           	}
           	else{
              	viCheckErr( viPrintf (io, "MEAS:VOLT?")); 
              	viCheckErr( viScanf (io, "%lf\n", &ACVOLT));
              	viCheckErr( viPrintf (io, "MEAS:CURR?"));   
              	viCheckErr( viScanf (io, "%lf", &ACCURR));  
          	}          
          
           	ACPOWER = ACVOLT * ACCURR;
           	*measurement = ACPOWER;
       	}
    }
    
    else if( measureFunction == CIC_RP50_VAL_FREQUENCY ){
       	ViSession     io = Ivi_IOSession(vi); // call only when locked
        
        if( serial_flag ){
           	viCheckErr( viPrintf (io, "FREQ?\n")); 
           	Delay(0.02);
           	viCheckErr( viScanf (io, "%lf\n", measurement));  
       		Delay(0.02);
       	}
        else{
           	viCheckErr( viPrintf (io, "FREQ?"));   
           	viCheckErr( viScanf (io, "%lf", measurement));  
       	}          
   	}
    
    else if( measureFunction == CIC_RP50_VAL_PEAK_CURRENT ){
       	ViSession     io = Ivi_IOSession(vi); // call only when locked
        
        if( model_number == 2001 || model_number == 2003 ){
           	if( serial_flag ){
              	viCheckErr( viPrintf (io, "MEAS:CURR:AMPL:MAX?\n")); 
              	Delay(0.02);
              	viCheckErr( viScanf (io, "%lf\n", measurement));  
           		Delay(0.02);
           	}
           	else{
              	viCheckErr( viPrintf (io, "MEAS:CURR:AMPL:MAX?"));   
              	viCheckErr( viScanf (io, "%lf", measurement));  
           	}
       	}
        else
   			return CIC_RP50_ERROR_NOT_SUPPORTED;  
   	}
    
    else if( measureFunction == CIC_RP50_VAL_CREST_FACTOR ){
       	ViSession     io = Ivi_IOSession(vi); // call only when locked
        
        if( model_number == 2001 || model_number == 2003 ){
           	if( serial_flag ){
              	viCheckErr( viPrintf (io, "MEAS:CURR:CRES?\n")); 
              	Delay(0.02);
              	viCheckErr( viScanf (io, "%lf\n", measurement));  
           		Delay(0.02);
           	}
           	else{
              	viCheckErr( viPrintf (io, "MEAS:CURR:CRES?"));   
              	viCheckErr( viScanf (io, "%lf", measurement));  
           	}          
       	}
        else
        	return CIC_RP50_ERROR_NOT_SUPPORTED;  
   	}
    
    else if( measureFunction == CIC_RP50_VAL_POWER ){
       	ViSession     io = Ivi_IOSession(vi); // call only when locked
        
        if( model_number == 2001 || model_number == 2003 ){
        	if( serial_flag ){
           		viCheckErr( viPrintf (io, "MEAS:POW?\n") ); 
           		Delay(0.02);
           		viCheckErr( viScanf (io, "%lf\n", measurement) );  
        		Delay(0.02);
        	}
        	else{
           		viCheckErr( viPrintf (io, "MEAS:POW?") );   
           		viCheckErr( viScanf (io, "%lf", measurement) );  
        	}          
   	    }
   	    else{
        	ViReal64 ACVOLT, ACCURR, ACPOWER;
        	if( serial_flag ){
           		viCheckErr( viPrintf(io, "MEAS:VOLT?\n") ); 
           		viCheckErr( viScanf(io, "%lf\n", &ACVOLT) );
           		viCheckErr( viPrintf(io, "MEAS:CURR?\n") ); 
           		viCheckErr( viScanf(io, "%lf\n", &ACCURR) );  
          	}
        	else{
           		viCheckErr( viPrintf(io, "MEAS:VOLT?") ); 
           		viCheckErr( viScanf(io, "%lf\n", &ACVOLT) );
           		viCheckErr( viPrintf(io, "MEAS:CURR?") );   
           		viCheckErr( viScanf(io, "%lf", &ACCURR) );  
          	}          
          
        	ACPOWER = ACVOLT * ACCURR;
        	*measurement = ACPOWER;
      	}
   	}
    
    else if( measureFunction == CIC_RP50_VAL_POWER_FACTOR ){
      	ViSession     io = Ivi_IOSession(vi); // call only when locked
        
        if( model_number == 2001 || model_number == 2003 ){
           	if( serial_flag ){
              	viCheckErr( viPrintf (io, "MEAS:POW:PFAC?\n")); 
              	viCheckErr( viScanf (io, "%lf\n", measurement));  
           		Delay(0.02);
           	}
           	else{
              	viCheckErr( viPrintf (io, "MEAS:POW:PFAC?"));   
              	viCheckErr( viScanf (io, "%lf", measurement));  
          	}          
        }
        else
        	return CIC_RP50_ERROR_NOT_SUPPORTED;  
   	}
     
Error:
    Ivi_UnlockSession( vi, VI_NULL );
    return error;
}

/*****************************************************************************
 * Function: CIC_RP50_GetAttribute<type> Functions                                    
 * Purpose:  These functions enable the instrument driver user to get 
 *           attribute values directly.  There are typesafe versions for 
 *           ViInt32, ViReal64, ViString, ViBoolean, and ViSession attributes.                                         
 *****************************************************************************/
ViStatus _VI_FUNC CIC_RP50_GetAttributeViInt32 (ViSession vi, ViConstString channelName, 
                                                ViAttr attributeId, ViInt32 *value)
{                                                                                                           
    return Ivi_GetAttributeViInt32 (vi, channelName, attributeId, IVI_VAL_DIRECT_USER_CALL, 
                                    value);
}                                                                                                           
ViStatus _VI_FUNC CIC_RP50_GetAttributeViReal64 (ViSession vi, ViConstString channelName, 
                                                 ViAttr attributeId, ViReal64 *value)
{                                                                                                           
    return Ivi_GetAttributeViReal64 (vi, channelName, attributeId, IVI_VAL_DIRECT_USER_CALL, 
                                     value);
}                                                                                                           
ViStatus _VI_FUNC CIC_RP50_GetAttributeViString (ViSession vi, ViConstString channelName, 
                                                 ViAttr attributeId, ViInt32 bufSize, 
                                                 ViChar value[]) 
{   
    return Ivi_GetAttributeViString (vi, channelName, attributeId, IVI_VAL_DIRECT_USER_CALL, 
                                     bufSize, value);
}   
ViStatus _VI_FUNC CIC_RP50_GetAttributeViBoolean (ViSession vi, ViConstString channelName, 
                                                  ViAttr attributeId, ViBoolean *value)
{                                                                                                           
    return Ivi_GetAttributeViBoolean (vi, channelName, attributeId, IVI_VAL_DIRECT_USER_CALL, 
                                      value);
}                                                                                                           
ViStatus _VI_FUNC CIC_RP50_GetAttributeViSession (ViSession vi, ViConstString channelName, 
                                                  ViAttr attributeId, ViSession *value)
{                                                                                                           
    return Ivi_GetAttributeViSession (vi, channelName, attributeId, IVI_VAL_DIRECT_USER_CALL, 
                                      value);
}                                                                                                           

/*****************************************************************************
 * Function: CIC_RP50_SetAttribute<type> Functions                                    
 * Purpose:  These functions enable the instrument driver user to set 
 *           attribute values directly.  There are typesafe versions for 
 *           ViInt32, ViReal64, ViString, ViBoolean, and ViSession datatypes.                                         
 *****************************************************************************/
ViStatus _VI_FUNC CIC_RP50_SetAttributeViInt32 (ViSession vi, ViConstString channelName, 
                                                ViAttr attributeId, ViInt32 value)
{                                                                                                           
    return Ivi_SetAttributeViInt32 (vi, channelName, attributeId, IVI_VAL_DIRECT_USER_CALL, 
                                    value);
}                                                                                                           
ViStatus _VI_FUNC CIC_RP50_SetAttributeViReal64 (ViSession vi, ViConstString channelName, 
                                                 ViAttr attributeId, ViReal64 value)
{                                                                                                           
    return Ivi_SetAttributeViReal64 (vi, channelName, attributeId, IVI_VAL_DIRECT_USER_CALL, 
                                     value);
}                                                                                                           
ViStatus _VI_FUNC CIC_RP50_SetAttributeViString (ViSession vi, ViConstString channelName, 
                                                 ViAttr attributeId, ViConstString value) 
{   
    return Ivi_SetAttributeViString (vi, channelName, attributeId, IVI_VAL_DIRECT_USER_CALL, 
                                     value);
}   
ViStatus _VI_FUNC CIC_RP50_SetAttributeViBoolean (ViSession vi, ViConstString channelName, 
                                                  ViAttr attributeId, ViBoolean value)
{                                                                                                           
    return Ivi_SetAttributeViBoolean (vi, channelName, attributeId, IVI_VAL_DIRECT_USER_CALL, 
                                      value);
}                                                                                                           
ViStatus _VI_FUNC CIC_RP50_SetAttributeViSession (ViSession vi, ViConstString channelName, 
                                                  ViAttr attributeId, ViSession value)
{                                                                                                           
    return Ivi_SetAttributeViSession (vi, channelName, attributeId, IVI_VAL_DIRECT_USER_CALL, 
                                      value);
}                                                                                                           

/*****************************************************************************
 * Function: CIC_RP50_CheckAttribute<type> Functions                                  
 * Purpose:  These functions enable the instrument driver user to check  
 *           attribute values directly.  These functions check the value you
 *           specify even if you set the CIC_RP50_ATTR_RANGE_CHECK 
 *           attribute to VI_FALSE.  There are typesafe versions for ViInt32, 
 *           ViReal64, ViString, ViBoolean, and ViSession datatypes.                         
 *****************************************************************************/
ViStatus _VI_FUNC CIC_RP50_CheckAttributeViInt32 (ViSession vi, ViConstString channelName, 
                                                  ViAttr attributeId, ViInt32 value)
{                                                                                                           
    return Ivi_CheckAttributeViInt32 (vi, channelName, attributeId, IVI_VAL_DIRECT_USER_CALL, 
                                      value);
}
ViStatus _VI_FUNC CIC_RP50_CheckAttributeViReal64 (ViSession vi, ViConstString channelName, 
                                                   ViAttr attributeId, ViReal64 value)
{                                                                                                           
    return Ivi_CheckAttributeViReal64 (vi, channelName, attributeId, IVI_VAL_DIRECT_USER_CALL, 
                                       value);
}                                                                                                           
ViStatus _VI_FUNC CIC_RP50_CheckAttributeViString (ViSession vi, ViConstString channelName, 
                                                   ViAttr attributeId, ViConstString value)  
{   
    return Ivi_CheckAttributeViString (vi, channelName, attributeId, IVI_VAL_DIRECT_USER_CALL, 
                                       value);
}   
ViStatus _VI_FUNC CIC_RP50_CheckAttributeViBoolean (ViSession vi, ViConstString channelName, 
                                                    ViAttr attributeId, ViBoolean value)
{                                                                                                           
    return Ivi_CheckAttributeViBoolean (vi, channelName, attributeId, IVI_VAL_DIRECT_USER_CALL, 
                                        value);
}                                                                                                           
ViStatus _VI_FUNC CIC_RP50_CheckAttributeViSession (ViSession vi, ViConstString channelName, 
                                                    ViAttr attributeId, ViSession value)
{                                                                                                           
    return Ivi_CheckAttributeViSession (vi, channelName, attributeId, IVI_VAL_DIRECT_USER_CALL, 
                                        value);
}                                                                                                           

/*****************************************************************************
 * Function: CIC_RP50_LockSession and CIC_RP50_UnlockSession Functions                        
 * Purpose:  These functions enable the instrument driver user to lock the 
 *           session around a sequence of driver calls during which other
 *           execution threads must not disturb the instrument state.
 *                                                                          
 *           NOTE:  The callerHasLock parameter must be a local variable 
 *           initialized to VI_FALSE and passed by reference, or you can pass 
 *           VI_NULL.                     
 *****************************************************************************/
ViStatus _VI_FUNC CIC_RP50_LockSession (ViSession vi, ViBoolean *callerHasLock)  
{                                              
   
    return Ivi_LockSession(vi,callerHasLock);      
}                                              

ViStatus _VI_FUNC CIC_RP50_UnlockSession (ViSession vi, ViBoolean *callerHasLock) 
{                                              
   
    return Ivi_UnlockSession(vi,callerHasLock);    
}   

/*****************************************************************************
 * Function: CIC_RP50_GetErrorInfo and CIC_RP50_ClearErrorInfo Functions                       
 * Purpose:  These functions enable the instrument driver user to  
 *           get or clear the error information the driver associates with the
 *           IVI session.                                                        
 *****************************************************************************/
ViStatus _VI_FUNC CIC_RP50_GetErrorInfo (ViSession vi, ViStatus *primaryError, 
                                         ViStatus *secondaryError, ViChar errorElaboration[256])  
{                                                                                                           
    return Ivi_GetErrorInfo(vi, primaryError, secondaryError, errorElaboration);                                
}                                                                                                           
ViStatus _VI_FUNC CIC_RP50_ClearErrorInfo (ViSession vi)                                                        
{                                                                                                           
    return Ivi_ClearErrorInfo (vi);                                                                             
}

/*****************************************************************************
 * Function: WriteInstrData and ReadInstrData Functions                      
 * Purpose:  These functions enable the instrument driver user to  
 *           write and read commands directly to and from the instrument.            
 *                                                                           
 *           Note:  These functions bypass the IVI attribute state caching.  
 *                  WriteInstrData invalidates the cached values for all 
 *                  attributes.
 *****************************************************************************/
ViStatus _VI_FUNC CIC_RP50_WriteInstrData (ViSession vi, ViConstString writeBuffer)   
{   
    return Ivi_WriteInstrData (vi, writeBuffer);    
}   
ViStatus _VI_FUNC CIC_RP50_ReadInstrData (ViSession vi, ViInt32 numBytes, 
                                          ViChar rdBuf[], ViInt32 *bytesRead)  
{   
    return Ivi_ReadInstrData (vi, numBytes, rdBuf, bytesRead);   
} 

/*****************************************************************************
 *-------------------- Utility Functions (Not Exported) ---------------------*
 *****************************************************************************/

/*****************************************************************************
 *  Function: CIC_RP50_OutputStatus
 *  Purpose:  This function gets the output function status.
 *****************************************************************************/
static ViStatus CIC_RP50_GetOutputStatus (ViSession vi,
                                          ViConstString channel,
                                          ViInt32 function, 
                                          ViReal64 *value)
{                          
    ViStatus      error = VI_SUCCESS;
    ViConstString coercedChannel;

    viCheckParm (Ivi_SetAttributeViInt32 (vi, channel,
                                          CIC_RP50_ATTR_QUERY_FUNCTION,
                                          0, function),
                                          3, "Query Function");
    checkErr( Ivi_LockSession(vi, VI_NULL));

    if (!Ivi_Simulating(vi))                  // call only when locked
          {
           ViSession     io = Ivi_IOSession(vi); // call only when locked
           ViConstString coercedChannel;

           checkErr( Ivi_CoerceChannelName (vi, channel, &coercedChannel));
           checkErr( Ivi_SetNeedToCheckStatus (vi, VI_TRUE)); 
           viCheckErr( viScanf (io, "%lf", value));
           checkErr( CIC_RP50_CheckStatus (vi));
           }
                            
    

    checkErr( Ivi_CoerceChannelName (vi, channel, &coercedChannel));

    checkErr( Ivi_SetNeedToCheckStatus (vi, VI_TRUE));

    

Error:
    Ivi_UnlockSession(vi, VI_NULL);
    return error;
}

/*****************************************************************************
 *  Function: CIC_RP50_SimulateMeasurement
 *  Purpose:  This function simulates an output measurement.
 *****************************************************************************/
static ViStatus CIC_RP50_SimulateMeasurement (ViSession vi,
                                              ViConstString channel,
                                              ViInt32 readingArraySize,
                                              ViReal64 readingArray[],
                                              ViInt32 *pointsRead,
                                              ViBoolean multipoint)
{                          
    ViStatus  error = VI_SUCCESS;
    ViStatus  status;
    ViReal64  sample;
    ViInt32   measureFunction;
    ViInt32   i;
    ViBoolean outputConnected;

    checkErr( Ivi_LockSession (vi, VI_NULL));

    *pointsRead = (multipoint) ? readingArraySize : 1;
                                      
    checkErr( Ivi_GetAttributeViBoolean (vi, channel, 
                                           CIC_RP50_ATTR_OUTPUT_CONNECTED,
                                           0, &outputConnected));
    for (i=0; i<*pointsRead; i++)
        {
        if (outputConnected)
            {
            switch (measureFunction) 
                {
                case CIC_RP50_VAL_AC_VOLTS:
                    status = Ivi_GetAttributeViReal64 (vi, channel, 
                                                       CIC_RP50_ATTR_ACV_AMPLITUDE,
                                                       0, &sample);
                    if (status != VI_SUCCESS)
                        sample = rand();
                    break;
                case CIC_RP50_VAL_AC_CURRENT:
                    status = Ivi_GetAttributeViReal64 (vi, channel, 
                                                       CIC_RP50_ATTR_ACC_AMPLITUDE,
                                                       0, &sample);
                    if (status != VI_SUCCESS)
                        sample = rand();
                    break;
                case CIC_RP50_VAL_AC_POWER:
                    sample = rand();
                    break;
                case CIC_RP50_VAL_FREQUENCY:
                    status = Ivi_GetAttributeViReal64 (vi, channel, 
                                                       CIC_RP50_ATTR_FREQUENCY,
                                                       0, &sample);
                    if (status != VI_SUCCESS)
                        sample = rand();
                    break;
                }
               }
        else
            sample = 0;
        readingArray[i] = sample;
        }
    
Error:
    Ivi_UnlockSession(vi, VI_NULL);
    return error;
}

/*****************************************************************************
 *  Function: CIC_RP50_GetDriverSetupOption
 *  Purpose:  Get the value of a DriverSetup option.  The driver setup string
 *            is part of the option string the user passes to the
 *            CIC_RP50_InitWithOptions function.  Specifically it is the part
 *            that follows "DriverSetup=".  This function assumes the 
 *            DriverSetup string has the following format:
 *              "OptionTag1 : OptionValue1; OptionTag2 : OptionValue2"
 *            Leading and trailing white space is removed from the option
 *            value and both the option tag and option value may contain
 *            white space.  The option tag is case sensitive.
 *
 *            Example:
 *
 *              "DriverSetup= Model : InstrX ; Special Option:Feature Y "
 *
 *              optionTag "Model"          returns optionValue "InstrX"
 *
 *              optionTag "Special Option" returns optionValue "Feature Y"
 *****************************************************************************/
static ViStatus CIC_RP50_GetDriverSetupOption (ViSession vi,
                                               ViConstString optionTag,
                                               ViInt32 optionValueSize,
                                               ViChar optionValue[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   setup[BUFFER_SIZE];
    ViChar*  searchOrigin;
    ViChar*  start;
    ViChar*  end;
    ViUInt32 n;

    checkErr( Ivi_LockSession (vi, VI_NULL));

    if ((optionValue != VI_NULL) && (optionValueSize > 0))
        optionValue[0] = 0;

    if ((optionTag != VI_NULL) && (optionValue != VI_NULL) && (optionValueSize > 1))
        {
        checkErr( Ivi_GetAttributeViString (vi, "", IVI_ATTR_DRIVER_SETUP,
                                            0, BUFFER_SIZE, setup));
        searchOrigin = setup;
        while (searchOrigin != VI_NULL)
            {
            searchOrigin = strstr(searchOrigin, optionTag);
            if (searchOrigin)
                {
                searchOrigin += strlen(optionTag);
                start = searchOrigin;
                while (isspace(*start)) start++;
                if (*start == ':')
                    {
                    start++;
                    while (isspace(*start)) start++;
                    if (*start)
                        {
                        end = strchr (start, ';');
                        if (start != end)
                            {
                            end = (end) ? end-- : &setup[strlen(setup)-1];
                            while (isspace(*end)) end--;
                            n = end - start + 1;
                            if (n > optionValueSize - 1)
                                n = optionValueSize - 1;
                            strncpy(optionValue, start, n);
                            optionValue[n] = 0;
                            break;
                            }
                        }
                    }
                }
            }
        }
        
Error:
    Ivi_UnlockSession (vi, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: CIC_RP50_CheckStatus                                                 
 * Purpose:  This function checks the status of the instrument to detect 
 *           whether the instrument has encountered an error.  This function  
 *           is called at the end of most exported driver functions.  If the    
 *           instrument reports an error, this function returns      
 *           IVI_ERROR_INSTR_SPECIFIC.  The user can set the 
 *           IVI_ATTR_QUERY_INSTR_STATUS attribute to VI_FALSE to disable this 
 *           check and increase execution speed.                                   
 *
 *           Note:  Call this function only when the session is locked.
 *****************************************************************************/
static ViStatus CIC_RP50_CheckStatus (ViSession vi)
{
    ViStatus    error = VI_SUCCESS;

    if (Ivi_QueryInstrStatus (vi) && Ivi_NeedToCheckStatus (vi) && !Ivi_Simulating (vi))
        {
        checkErr( CIC_RP50_CheckStatusCallback (vi, Ivi_IOSession(vi)));
        checkErr( Ivi_SetNeedToCheckStatus (vi, VI_FALSE));
        }
        
Error:
    return error;
}


/*****************************************************************************
 * Function: CIC_RP50_DefaultInstrSetup                                               
 * Purpose:  This function sends a default setup to the instrument.  The    
 *           CIC_RP50_reset function calls this function.  The 
 *           CIC_RP50_IviInit function calls this function when the
 *           user passes VI_FALSE for the reset parameter.  This function is 
 *           useful for configuring settings that other instrument driver 
 *           functions require.    
 *
 *           Note:  Call this function only when the session is locked.
 *****************************************************************************/
static ViStatus CIC_RP50_DefaultInstrSetup (ViSession vi)
{
    ViStatus    error = VI_SUCCESS;
        
    /* Invalidate all attributes */
    checkErr( Ivi_InvalidateAllAttributes (vi));
    

    if (!Ivi_Simulating(vi))
        {
        ViSession   io = Ivi_IOSession(vi); /* call only when locked */

        checkErr( Ivi_SetNeedToCheckStatus (vi, VI_TRUE));
		
		if (serial_flag)
		  {
		   viCheckErr( viPrintf (io, "*CLS\n"));
		  }
		else
		  viCheckErr( viPrintf (io, "*CLS"));  
		}
    	Delay(0.03);  
Error:
    return error;
}

/*****************************************************************************
 * Function: ReadToFile and WriteFromFile Functions                          
 * Purpose:  Functions for instrument driver developers to read/write        
 *           instrument data to/from a file.                                 
 *****************************************************************************/
static ViStatus CIC_RP50_ReadToFile (ViSession vi, ViConstString filename, 
                                     ViInt32 maxBytesToRead, ViInt32 fileAction, 
                                     ViInt32 *totalBytesWritten)  
{   
    return Ivi_ReadToFile (vi, filename, maxBytesToRead, fileAction, totalBytesWritten);  
}   
static ViStatus CIC_RP50_WriteFromFile (ViSession vi, ViConstString filename, 
                                        ViInt32 maxBytesToWrite, ViInt32 byteOffset, 
                                        ViInt32 *totalBytesWritten) 
{   
    return Ivi_WriteFromFile (vi, filename, maxBytesToWrite, byteOffset, totalBytesWritten); 
}

/*****************************************************************************
 *------------------------ Global Session Callbacks -------------------------*
 *****************************************************************************/

/*****************************************************************************
 * Function: CIC_RP50_CheckStatusCallback                                               
 * Purpose:  This function queries the instrument to determine if it has 
 *           encountered an error.  If the instrument has encountered an 
 *           error, this function returns the IVI_ERROR_INSTRUMENT_SPECIFIC 
 *           error code.  This function is called by the 
 *           CIC_RP50_CheckStatus utility function.  The 
 *           Ivi_SetAttribute and Ivi_GetAttribute functions invoke this 
 *           function when the optionFlags parameter includes the
 *           IVI_VAL_DIRECT_USER_CALL flag.
 *           
 *           The user can disable calls to this function by setting the 
 *           IVI_ATTR_QUERY_INSTR_STATUS attribute to VI_FALSE.  The driver can 
 *           disable the check status callback for a particular attribute by 
 *           setting the IVI_VAL_DONT_CHECK_STATUS flag.
 *****************************************************************************/
static ViStatus _VI_FUNC CIC_RP50_CheckStatusCallback (ViSession vi, ViSession io)
{
    ViStatus    error = VI_SUCCESS;
	
	
	/* Query instrument status */
	
	    ViInt32     esr = 0; 
	    if (serial_flag)
	      {
	       viCheckErr( viPrintf (io, "SYST:ERR?\n"));
	       Delay (.05);
	       viCheckErr( viScanf (io, "%hd\n", &esr));
	       Delay (.05);
	      }
		else
		  {
		   viCheckErr( viPrintf (io, "SYST:ERR?"));
		   Delay (.05);
		   viCheckErr( viScanf (io, "%hd", &esr));
		   Delay (.05);
		  }
		  
	    
	
	    if ((esr & IEEE_488_2_QUERY_ERROR_BIT) ||
	        (esr & IEEE_488_2_DEVICE_DEPENDENT_ERROR_BIT) ||
	        (esr & IEEE_488_2_EXECUTION_ERROR_BIT) ||
	        (esr & IEEE_488_2_COMMAND_ERROR_BIT))
	            {     
	            viCheckErr( IVI_ERROR_INSTR_SPECIFIC);
	            }	   

Error:
    return error;
}

/*****************************************************************************
 * Function: CIC_RP50_WaitForOPCCallback                                               
 * Purpose:  This function waits until the instrument has finished processing 
 *           all pending operations.  This function is called by the 
 *           CIC_RP50_WaitForOPC utility function.  The IVI engine invokes
 *           this function in the following two cases:
 *           - Before invoking the read callback for attributes for which the 
 *             IVI_VAL_WAIT_FOR_OPC_BEFORE_READS flag is set.
 *           - After invoking the write callback for attributes for which the 
 *             IVI_VAL_WAIT_FOR_OPC_AFTER_WRITES flag is set.
 *****************************************************************************/
static ViStatus _VI_FUNC CIC_RP50_WaitForOPCCallback (ViSession vi, ViSession io)
{
    ViStatus    error = VI_SUCCESS;
   

    return error;
}

/*****************************************************************************
 *----------------- Attribute Range Tables and Callbacks --------------------*
 *****************************************************************************/
/*- CIC_RP50_ATTR_ID_QUERY_RESPONSE -*/
static ViStatus _VI_FUNC CIC_RP50AttrIdQueryResponse_ReadCallback (ViSession vi, 
                                                                   ViSession io,
                                                                   ViConstString channelName, 
                                                                   ViAttr attributeId,
                                                                   const ViConstString cacheValue)
{
   	//ViChar      rdBuffers[BUFFER_SIZE];
    ViStatus    error = VI_SUCCESS;
    ViUInt32    retCnt;
    unsigned char  rdBuffer[BUFFER_SIZE];
    
    if( serial_flag ){
       	viCheckErr( viPrintf(io, "*IDN?\n") );						 
       	Delay(.05);
    }
    else
      	viCheckErr(viPrintf (io, "*IDN?") );
      	
    Delay(0.05);
    
    viCheckErr( viRead(io, rdBuffer, BUFFER_SIZE-1, &retCnt) );
    
    rdBuffer[retCnt] = 0;

    checkErr( Ivi_SetValInStringCallback(vi, attributeId, rdBuffer) );
    
Error:
    return error;
}
	
/**********************************************************************************************/    
/*- CIC_RP50_ATTR_DRIVER_REVISION -*/
static ViStatus _VI_FUNC CIC_RP50AttrDriverRevision_ReadCallback (ViSession vi, 
                                                                  ViSession io,
                                                                  ViConstString channelName, 
                                                                  ViAttr attributeId,
                                                                  const ViConstString cacheValue)
{
    ViStatus    error = VI_NULL;
    ViChar      driverRevision[256];
    
    
    sprintf (driverRevision, 
             "Driver: CIC_RP50 %d.%d, Compiler: %s %3.2f, "
             "Components: IVIEngine %d.%d, VISA-Spec %.2f",
             CIC_RP50_MAJOR_VERSION, CIC_RP50_MINOR_VERSION, 
             IVI_COMPILER_NAME, IVI_COMPILER_VER_NUM, IVI_ENGINE_MAJOR_VERSION, 
             IVI_ENGINE_MINOR_VERSION, Ivi_ConvertVISAVer(VI_SPEC_VERSION));

    viCheckErr( Ivi_SetValInStringCallback (vi, attributeId, driverRevision));    
Error:
    return error;
}

/*****************************************************************************/
/*- CIC_RP50_ATTR_FREQUENCY_LOW_LIMIT -*/
static ViStatus _VI_FUNC CIC_RP50AttrFrequencyLowLimit_ReadCallback (ViSession vi,
                                                                     ViSession io,
                                                                     ViConstString channelName,
                                                                     ViAttr attributeId,
                                                                     ViReal64 *value)
{
    ViStatus error = VI_SUCCESS;

    
    if (serial_flag)
      {
       viCheckErr (viPrintf (io, "LIM:FREQ:LOW?\n")); 
       viCheckErr (viScanf (io, "%lf\n", value)); 
      }
    else
      {
       viCheckErr (viPrintf (io, "LIM:FREQ:LOW?")); 
       viCheckErr (viScanf (io, "%lf", value)); 
      }
      
    
   
    
Error:
    return error;
}

/**********************************************************************************************/
/*- CIC_RP50_ATTR_FREQUENCY_HIGH_LIMIT -*/
static ViStatus _VI_FUNC CIC_RP50AttrFrequencyHighLimit_ReadCallback (ViSession vi,
                                                                      ViSession io,
                                                                      ViConstString channelName,
                                                                      ViAttr attributeId,
                                                                      ViReal64 *value)
{
    ViStatus error = VI_SUCCESS;

    
    if (serial_flag) 
      {
       viCheckErr (viPrintf (io, "LIM:FREQ:HIGH?\n"));   
       viCheckErr (viScanf (io, "%lf\n", value)); 
      }
    else
      {
       viCheckErr (viPrintf (io, "LIM:FREQ:HIGH?")); 
       viCheckErr (viScanf (io, "%lf", value)); 
      }
   
Error:
    return error;
}

/*- CIC_RP50_ATTR_VOLTAGE_LIMIT -*/

static IviRangeTableEntry attrVoltageLimitRangeTableEntries[] =
	{
		{0.0, 300.0, 0, "", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};
   
static IviRangeTable attrVoltageLimitRangeTable =
	{
		IVI_VAL_RANGED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        attrVoltageLimitRangeTableEntries,
	};

/*****************************************************************************/
static ViStatus _VI_FUNC CIC_RP50AttrVoltageLimit_ReadCallback (ViSession vi, 
                                                                ViSession io, 
                                                                ViConstString channelName, 
                                                                ViAttr attributeId, 
                                                                ViReal64 *value)
{
    ViStatus error = VI_SUCCESS;

    
    if (serial_flag)
      {
       viCheckErr (viPrintf (io, "LIM:VOLT?\n"));   
       viCheckErr (viScanf (io, "%lf\n", value));   
      }
    else
      {
       viCheckErr (viPrintf (io, "LIM:VOLT?"));
       viCheckErr (viScanf (io, "%lf", value));   
       }
    Delay(0.01);
    
Error:
    return error;
}

/*- CIC_RP50_ATTR_CURRENT_LIMIT -*/

static IviRangeTableEntry attrCurrentLimitRangeTableEntries[] =
	{
		{0.0, 13.34, 0, "", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};
   
static IviRangeTable attrCurrentLimitRangeTable =
	{
		IVI_VAL_RANGED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        attrCurrentLimitRangeTableEntries,
	};

/*****************************************************************************/
static ViStatus _VI_FUNC CIC_RP50AttrCurrentLimit_ReadCallback (ViSession vi, 
                                                                ViSession io, 
                                                                ViConstString channelName, 
                                                                ViAttr attributeId, 
                                                                ViReal64 *value)
{
    ViStatus error = VI_SUCCESS;

    
    if (serial_flag)
      {
       viCheckErr (viPrintf (io, "LIM:CURR?\n"));   
       viCheckErr (viScanf (io, "%lf\n", value));
      }
    else
      {
       viCheckErr (viPrintf (io, "LIM:CURR?"));    
       viCheckErr (viScanf (io, "%lf", value));
      }
       
Error:
    return error;
}

/*- CIC_RP50_ATTR_OUTPUT_CONNECTED -*/

static ViStatus _VI_FUNC CIC_RP50AttrOutputConnected_CheckCallback (ViSession vi,
                                                                    ViConstString channelName,
                                                                    ViAttr attributeId,
                                                                    ViBoolean value)
{
    ViStatus  error = VI_SUCCESS;
    
Error:
    return error;
}

/*****************************************************************************/
static ViStatus _VI_FUNC CIC_RP50AttrOutputConnected_WriteCallback (ViSession vi, 
                                                                    ViSession io, 
                                                                    ViConstString channelName, 
                                                                    ViAttr attributeId, 
                                                                    ViBoolean value)
{
    ViStatus error = VI_SUCCESS;

    if (serial_flag)
      {
       viCheckErr (viPrintf (io, "OUTP %ld\n", (value == VI_TRUE) ? 1 : 0));
      }
    else
      viCheckErr (viPrintf (io, "OUTP %ld", (value == VI_TRUE) ? 1 : 0));
    
    Delay(.03);  
Error:
    return error;
}

/*****************************************************************************/
static ViStatus _VI_FUNC CIC_RP50AttrOutputConnected_ReadCallback (ViSession vi, 
                                                                   ViSession io, 
                                                                   ViConstString channelName, 
                                                                   ViAttr attributeId, 
                                                                   ViBoolean *value)
{
    ViStatus error = VI_SUCCESS;

    
    ViInt32  state;

    if (serial_flag)
      {
       viCheckErr (viPrintf (io, "OUTP?\n"));   
       viCheckErr (viScanf (io, "%ld\n", &state));
      }
    else
      {
       viCheckErr (viPrintf (io, "OUTP?"));
       viCheckErr (viScanf (io, "%ld", &state));
      }
      
    
    *value = (state) ? VI_TRUE : VI_FALSE;   
    
    
Error:
    return error;
}


/*- CIC_RP50_ATTR_FREQUENCY -*/

static IviRangeTableEntry attrFrequencyRangeTableEntries[] =
	{
		{16.0, 5000.0, 0, "", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};
   
static IviRangeTable attrFrequencyRangeTable =
	{
		IVI_VAL_RANGED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        attrFrequencyRangeTableEntries,
	};

/*****************************************************************************/
static ViStatus _VI_FUNC CIC_RP50AttrFrequency_WriteCallback (ViSession vi, 
                                                              ViSession io, 
                                                              ViConstString channelName, 
                                                              ViAttr attributeId, 
                                                              ViReal64 value)
{
    ViStatus error = VI_SUCCESS;

    
    if (serial_flag)
      viCheckErr (viPrintf (io, "FREQ %lf\n", value));   
    else
      viCheckErr (viPrintf (io, "FREQ %lf", value));    
  
  	Delay(.03);  
Error:
    return error;
}

/*****************************************************************************/
static ViStatus _VI_FUNC CIC_RP50AttrFrequency_ReadCallback (ViSession vi, 
                                                             ViSession io, 
                                                             ViConstString channelName, 
                                                             ViAttr attributeId, 
                                                             ViReal64 *value)
{
    ViStatus error = VI_SUCCESS;
   
    if (serial_flag)
      {
       viCheckErr (viPrintf (io, "FREQ?\n"));   
       viCheckErr (viScanf (io, "%lf\n", value));   
      }
    else
      {
       viCheckErr (viPrintf (io, "FREQ?")); 
       viCheckErr (viScanf (io, "%lf", value));   
      }
    
Error:
    return error;
}

/*****************************************************************************/
/*- CIC_RP50_ATTR_VOLTAGE_RANGE -*/
static ViStatus _VI_FUNC CIC_RP50AttrACVRange_WriteCallback (ViSession vi, 
                                                             ViSession io, 
                                                             ViConstString channelName, 
                                                             ViAttr attributeId, 
                                                             ViReal64 value)
{
    ViStatus error = VI_SUCCESS;
	
   	if( serial_flag ){
      	if( value == 1.0 ){
          	viCheckErr(viPrintf(io, "VOLT:RANG %lf\n", volt_high_limit/2) ); 
       	}
       	else{
          	viCheckErr(viPrintf(io, "VOLT:RANG %lf\n",  volt_high_limit) );   
        }
   	}
    else{
       	if( value == 1.0 ){
          	viCheckErr(viPrintf(io, "VOLT:RANG %lf", volt_high_limit/2) );   
        }
       	else{
          	viCheckErr(viPrintf(io, "VOLT:RANG %lf",  volt_high_limit) );   
        }
    }
    Delay(.03);  

Error:
    return error;
}

/*****************************************************************************/
static ViStatus _VI_FUNC CIC_RP50AttrACVRange_ReadCallback (ViSession vi, 
                                                                ViSession io, 
                                                                ViConstString channelName, 
                                                                ViAttr attributeId, 
                                                                ViReal64 *value)
{
    ViStatus error = VI_SUCCESS;

   
    if (serial_flag)
      {
       viCheckErr (viPrintf (io, "VOLT:RANG?\n"));   
       viCheckErr (viScanf (io, "%lf\n", value));   
      }
    else
      {
       viCheckErr (viPrintf (io, "VOLT:RANG?"));  
       viCheckErr (viScanf (io, "%lf", value));   
      }
    
Error:
    return error;
}



/*- CIC_RP50_ATTR_ACV_AMPLITUDE -*/

static IviRangeTableEntry attrACVAmplitudeRangeTableEntries[] =
	{
		{0.0, 300.0, 0, "", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};
   
static IviRangeTable attrACVAmplitudeRangeTable =
	{
		IVI_VAL_RANGED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        attrACVAmplitudeRangeTableEntries,
	};

/*****************************************************************************/
static ViStatus _VI_FUNC CIC_RP50AttrACVAmplitude_WriteCallback (ViSession vi, 
                                                                 ViSession io, 
                                                                 ViConstString channelName, 
                                                                 ViAttr attributeId, 
                                                                 ViReal64 value)
{
    ViStatus error = VI_SUCCESS;

   

    if (serial_flag)
      {
       viCheckErr (viPrintf (io, "VOLT %lf\n", value));   
      }
    else
      viCheckErr (viPrintf (io, "VOLT %lf", value)); 
    
    Delay(.03);  
Error:
    return error;
}

/*****************************************************************************/
static ViStatus _VI_FUNC CIC_RP50AttrACVAmplitude_ReadCallback (ViSession vi, 
                                                                ViSession io, 
                                                                ViConstString channelName, 
                                                                ViAttr attributeId, 
                                                                ViReal64 *value)
{
    ViStatus error = VI_SUCCESS;

    
    if (serial_flag)
      {
       viCheckErr (viPrintf (io, "VOLT?\n"));   
       viCheckErr (viScanf (io, "%lf\n", value)); 
      }
    else
      {
       viCheckErr (viPrintf (io, "VOLT?")); 
       viCheckErr (viScanf (io, "%lf", value)); 
      }
      
Error:
    return error;
}

/*- CIC_RP50_ATTR_ACC_AMPLITUDE -*/

static IviRangeTableEntry attrACCAmplitudeRangeTableEntries[] =
	{
		{0.0, 13.34, 0, "", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};
   
static IviRangeTable attrACCAmplitudeRangeTable =
	{
		IVI_VAL_RANGED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        attrACCAmplitudeRangeTableEntries,
	};

/*****************************************************************************/
static ViStatus _VI_FUNC CIC_RP50AttrACCAmplitude_WriteCallback (ViSession vi, 
                                                                 ViSession io, 
                                                                 ViConstString channelName, 
                                                                 ViAttr attributeId, 
                                                                 ViReal64 value)
{
    ViStatus error = VI_SUCCESS;

   
    if (serial_flag)
      viCheckErr (viPrintf (io, "CURR %lf\n", value));   
    else
      viCheckErr (viPrintf (io, "CURR %lf", value));    
    
    Delay(.03);  
Error:
    return error;
}

/*****************************************************************************/
static ViStatus _VI_FUNC CIC_RP50AttrACCAmplitude_ReadCallback (ViSession vi, 
                                                                ViSession io, 
                                                                ViConstString channelName, 
                                                                ViAttr attributeId, 
                                                                ViReal64 *value)
{
    ViStatus error = VI_SUCCESS;

   

    if (serial_flag)
      {
       viCheckErr (viPrintf (io, "CURR?\n"));   
       viCheckErr (viScanf (io, "%lf\n", value));
      }
    else
      {
       viCheckErr (viPrintf (io, "CURR?"));
       viCheckErr (viScanf (io, "%lf", value));
      }
      
    viCheckErr (viScanf (io, "%lf", value));   
    
Error:
    return error;
}

/*- CIC_RP50_ATTR_QUERY_FUNCTION -*/

static IviRangeTableEntry attrQueryFunctionRangeTableEntries[] =
	{
		{CIC_RP50_VAL_FREQUENCY, 0, 0, "FREQ", 0},
		{CIC_RP50_VAL_VOLTS, 0, 0, "VOLT", 0},
		{CIC_RP50_VAL_CURRENT, 0, 0, "CURR", 0},
		{CIC_RP50_VAL_RANGE, 0, 0, "VOLT:RANG", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};
   
static IviRangeTable attrQueryFunctionRangeTable =
	{
		IVI_VAL_DISCRETE,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        attrQueryFunctionRangeTableEntries,
	};

/*****************************************************************************/
static ViStatus _VI_FUNC CIC_RP50AttrQueryFunction_ReadCallback (ViSession vi,
                                                                 ViSession io,
                                                                 ViConstString channelName,
                                                                 ViAttr attributeId,
                                                                 ViInt32 *value){
    ViStatus error = VI_SUCCESS;

   
Error:
    return error;
}

/*****************************************************************************/
static ViStatus _VI_FUNC CIC_RP50AttrQueryFunction_WriteCallback (ViSession vi,
                                                                  ViSession io,
                                                                  ViConstString channelName,
                                                                  ViAttr attributeId,
                                                                  ViInt32 value){
    ViStatus      error = VI_SUCCESS;

   
    ViString cmd;                                                                                                                     
                                                                                                                                
    checkErr( Ivi_GetViInt32EntryFromValue (value, &attrQueryFunctionRangeTable, VI_NULL,
                                            VI_NULL, VI_NULL, VI_NULL, &cmd, VI_NULL));         
   
    if (serial_flag)
      viCheckErr( viPrintf (io, "%s?\n", cmd));                                                                                              
    else
      viCheckErr( viPrintf (io, "%s?", cmd));
    
Error:
    return error;
}

/*****************************************************************************/
/*- CIC_RP50_ATTR_REMOTE_ON_OFF -*/     
static ViStatus _VI_FUNC CIC_RP50AttrRemoteOnOff_CheckCallback (ViSession vi,
                                                                ViConstString channelName,
                                                                ViAttr attributeId,
                                                                ViInt32 value)
{
	ViStatus	error = VI_SUCCESS;
	
Error:
	return error;
}

static ViStatus _VI_FUNC CIC_RP50AttrRemoteOnOff_ReadCallback (ViSession vi,
                                                               ViSession io,
                                                               ViConstString channelName,
                                                               ViAttr attributeId,
                                                               ViInt32 *value)
{
	ViStatus	error = VI_SUCCESS;

Error:
	return error;
}

/*****************************************************************************/
static ViStatus _VI_FUNC CIC_RP50AttrRemoteOnOff_WriteCallback (ViSession vi,
                                                                ViSession io,
                                                                ViConstString channelName,
                                                                ViAttr attributeId,
                                                                ViInt32 value)
{
	ViStatus	error = VI_SUCCESS;
	
	if (serial_flag)
	  {
	   if (value == 1)
         {
          viCheckErr (viPrintf (io, "SYST:REM\n"));
         }
       else 
         viCheckErr (viPrintf (io, "SYST:LOC\n"));
       }
     else
      {
       if (value == 1)
         viCheckErr (viPrintf (io, "SYST:REM"));
       else 
         viCheckErr (viPrintf (io, "SYST:LOC"));
       }
     
     Delay(.03);  
     
Error:
	return error;
}

/*****************************************************************************/
/*- CIC_RP50_ATTR_DISPLAY_MODE -*/   

static ViStatus _VI_FUNC CIC_RP50AttrDisplayMode_CheckCallback (ViSession vi,
                                                                ViConstString channelName,
                                                                ViAttr attributeId,
                                                                ViInt32 value)
{
	ViStatus	error = VI_SUCCESS;

Error:
	return error;
}

/*****************************************************************************/
static ViStatus _VI_FUNC CIC_RP50AttrDisplayMode_ReadCallback (ViSession vi,
                                                               ViSession io,
                                                               ViConstString channelName,
                                                               ViAttr attributeId,
                                                               ViInt32 *value)
{
	ViStatus	error = VI_SUCCESS;

Error:
	return error;
}

/*****************************************************************************/
static ViStatus _VI_FUNC CIC_RP50AttrDisplayMode_WriteCallback (ViSession vi,
                                                                ViSession io,
                                                                ViConstString channelName,
                                                                ViAttr attributeId,
                                                                ViInt32 value)
{
	ViStatus	error = VI_SUCCESS;
	
	if( serial_flag ){
	  	if( model_number == 2001 || model_number == 2003 ){
          	viCheckErr (viPrintf (io, "DISP:MODE %ld\n", value));
      	}
       	else{
         	if( value == 2 ){
             	viCheckErr (viPrintf (io, "DISP:MODE 0\n"));
            }
	      	else if( value == 1 ){
             	viCheckErr (viPrintf (io, "DISP:MODE 1\n"));
           	}
       	}
   	}
    else{
	  	if( model_number == 2001 || model_number == 2003 ){
          	viCheckErr (viPrintf (io, "DISP:MODE %ld", value));
        }
       	else{
          	if( value == 2 ){
             	viCheckErr (viPrintf (io, "DISP:MODE 0"));
            }
	      	else if( value == 1 ){
             	viCheckErr (viPrintf (io, "DISP:MODE 1"));
           	}
      	}
  	}
    Delay(.03);   
Error:
	return error;
}

/*****************************************************************************
 * Function: CIC_RP50_InitAttributes                                                  
 * Purpose:  This function adds attributes to the IVI session, initializes   
 *           instrument attributes, and sets attribute invalidation          
 *           dependencies.                                                   
 *****************************************************************************/
static ViStatus CIC_RP50_InitAttributes (ViSession vi)
{
    ViStatus    error = VI_SUCCESS;
       
        /*- Initialize instrument attributes --------------------------------*/            

    checkErr( Ivi_SetAttributeViInt32 (vi, VI_NULL, CIC_RP50_ATTR_DRIVER_MAJOR_VERSION, 
                                       0, CIC_RP50_MAJOR_VERSION));                
    
    checkErr( Ivi_SetAttributeViInt32 (vi, VI_NULL, CIC_RP50_ATTR_DRIVER_MINOR_VERSION, 
                                       0, CIC_RP50_MINOR_VERSION));                
    
    checkErr( Ivi_SetAttrReadCallbackViString (vi, CIC_RP50_ATTR_DRIVER_REVISION,          
                                               CIC_RP50AttrDriverRevision_ReadCallback));

    checkErr( Ivi_SetAttributeViAddr (vi, VI_NULL, IVI_ATTR_OPC_CALLBACK, 0,
                                      CIC_RP50_WaitForOPCCallback));
    
    checkErr( Ivi_SetAttributeViAddr (vi, VI_NULL, IVI_ATTR_CHECK_STATUS_CALLBACK, 0,
                                      CIC_RP50_CheckStatusCallback));
	    
	checkErr( Ivi_SetAttributeViBoolean (vi, VI_NULL, IVI_ATTR_SUPPORTS_WR_BUF_OPER_MODE, 
	                                     0, VI_TRUE));

        /*- Add instrument-specific attributes ------------------------------*/            
	    
	checkErr (Ivi_AddAttributeViReal64 (vi, CIC_RP50_ATTR_VOLTAGE_LIMIT,
	                                    "CIC_RP50_ATTR_VOLTAGE_LIMIT", 300.0,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    CIC_RP50AttrVoltageLimit_ReadCallback,
	                                    VI_NULL, &attrVoltageLimitRangeTable, 0));
	
	checkErr (Ivi_AddAttributeViReal64 (vi, CIC_RP50_ATTR_CURRENT_LIMIT,
	                                    "CIC_RP50_ATTR_CURRENT_LIMIT", 9.2,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    CIC_RP50AttrCurrentLimit_ReadCallback,
	                                    VI_NULL, &attrCurrentLimitRangeTable, 0));
	
	checkErr (Ivi_AddAttributeViReal64 (vi, CIC_RP50_ATTR_FREQUENCY_LOW_LIMIT,
	                                    "CIC_RP50_ATTR_FREQUENCY_LOW_LIMIT", 16.0,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    CIC_RP50AttrFrequencyLowLimit_ReadCallback,
	                                    VI_NULL, &attrFrequencyRangeTable, 0));
	
	checkErr (Ivi_AddAttributeViReal64 (vi, CIC_RP50_ATTR_FREQUENCY_HIGH_LIMIT,
	                                    "CIC_RP50_ATTR_FREQUENCY_HIGH_LIMIT", 50.0,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    CIC_RP50AttrFrequencyHighLimit_ReadCallback,
	                                    VI_NULL, &attrFrequencyRangeTable, 0));
	
	checkErr( Ivi_AddAttributeViBoolean (vi, CIC_RP50_ATTR_OUTPUT_CONNECTED,
	                                     "CIC_RP50_ATTR_OUTPUT_CONNECTED",
	                                     VI_TRUE, IVI_VAL_MULTI_CHANNEL | IVI_VAL_NEVER_CACHE,
	                                     CIC_RP50AttrOutputConnected_ReadCallback,
	                                     CIC_RP50AttrOutputConnected_WriteCallback));
	
	checkErr (Ivi_SetAttrCheckCallbackViBoolean (vi, CIC_RP50_ATTR_OUTPUT_CONNECTED,
	                                             CIC_RP50AttrOutputConnected_CheckCallback));
	
	checkErr (Ivi_AddAttributeViReal64 (vi, CIC_RP50_ATTR_FREQUENCY,
	                                    "CIC_RP50_ATTR_FREQUENCY", 16.0,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    CIC_RP50AttrFrequency_ReadCallback,
	                                    CIC_RP50AttrFrequency_WriteCallback,
	                                    &attrFrequencyRangeTable, 0));
	
	checkErr( Ivi_AddAttributeViReal64  (vi, CIC_RP50_ATTR_ACV_AMPLITUDE,
	                                     "CIC_RP50_ATTR_ACV_AMPLITUDE",
	                                     0.0, IVI_VAL_MULTI_CHANNEL,
	                                     CIC_RP50AttrACVAmplitude_ReadCallback,
	                                     CIC_RP50AttrACVAmplitude_WriteCallback,
	                                     &attrACVAmplitudeRangeTable, 0));
	
	checkErr( Ivi_AddAttributeViReal64  (vi, CIC_RP50_ATTR_ACC_AMPLITUDE,
	                                     "CIC_RP50_ATTR_ACC_AMPLITUDE",
	                                     0.0, IVI_VAL_MULTI_CHANNEL,
	                                     CIC_RP50AttrACCAmplitude_ReadCallback,
	                                     CIC_RP50AttrACCAmplitude_WriteCallback,
	                                     &attrACCAmplitudeRangeTable, 0));
	
	checkErr (Ivi_AddAttributeViInt32 (vi, CIC_RP50_ATTR_QUERY_FUNCTION,
	                                   "CIC_RP50_ATTR_QUERY_FUNCTION",
	                                   CIC_RP50_VAL_AC_FREQUENCY,
	                                   IVI_VAL_MULTI_CHANNEL,
	                                   CIC_RP50AttrQueryFunction_ReadCallback,
	                                   CIC_RP50AttrQueryFunction_WriteCallback,
	                                   &attrQueryFunctionRangeTable));
	
	checkErr( Ivi_AddAttributeViString (vi, CIC_RP50_ATTR_ID_QUERY_RESPONSE,
	                                    "CIC_RP50_ATTR_ID_QUERY_RESPONSE", 
	                                    "CI",
	                                    IVI_VAL_NOT_USER_WRITABLE,
	                                    CIC_RP50AttrIdQueryResponse_ReadCallback,
	                                    VI_NULL));
	
	checkErr (Ivi_AddAttributeViReal64 (vi, CIC_RP50_ATTR_VOLTAGE_RANGE,
	                                    "CIC_RP50_ATTR_VOLTAGE_RANGE", 136.0,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    CIC_RP50AttrACVRange_ReadCallback,
	                                    CIC_RP50AttrACVRange_WriteCallback,
	                                    &attrVoltageLimitRangeTable, 0));
	
	checkErr (Ivi_AddAttributeViInt32 (vi, CIC_RP50_ATTR_REMOTE_ON_OFF,
	                                   "CIC_RP50_ATTR_REMOTE_ON_OFF", 0,
	                                   IVI_VAL_MULTI_CHANNEL,
	                                   CIC_RP50AttrRemoteOnOff_ReadCallback,
	                                   CIC_RP50AttrRemoteOnOff_WriteCallback,
	                                   VI_NULL));
	
	checkErr (Ivi_AddAttributeViInt32 (vi, CIC_RP50_ATTR_DISPLAY_MODE,
	                                   "CIC_RP50_ATTR_DISPLAY_MODE", 0,
	                                   IVI_VAL_MULTI_CHANNEL,
	                                   CIC_RP50AttrDisplayMode_ReadCallback,
	                                   CIC_RP50AttrDisplayMode_WriteCallback,
	                                   VI_NULL));
	
	checkErr (Ivi_SetAttrCheckCallbackViInt32 (vi, CIC_RP50_ATTR_DISPLAY_MODE,
	                                           CIC_RP50AttrDisplayMode_CheckCallback));
	
	checkErr (Ivi_SetAttrCheckCallbackViInt32 (vi, CIC_RP50_ATTR_REMOTE_ON_OFF,
	                                           CIC_RP50AttrRemoteOnOff_CheckCallback));
	                                           
    
        /*- Setup attribute invalidations -----------------------------------*/            
	
Error:
    return error;
}

/*****************************************************************************
 *------------------- End Instrument Driver Source Code ---------------------*
 *****************************************************************************/
 
//v1.0.0.2 		10/25/05 
//Reduce rs232 timeout time to 10sec
